#!/usr/bin/env python

######################################################################
#  Copyright (c) 2000, Sean Reifschneider, tummy.com, ltd.
#  All Rights Reserved
#
#  High-level classes for implementing command/response clients and
#  servers in Python.  Though the server is much more highly developed
#  than the client at this point.
#
#  ftp://ftp.tummy.com/pub/tummy/cccheck/
######################################################################
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.0 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is cccheck.py, released October 16, 2000
# The Initial Developer of the Original Code is tummy.com, ltd.
# Portions created by tummy.com, ltd. are Copyright (C) 2000
# tummy.com, ltd. All Rights Reserved.
#
# Contributor(s): ______________________________________. 
#
######################################################################


revision = "$Revision: 1.1.1.1 $"
rcsid = "$Id: cccheck.py,v 1.1.1.1 2000/10/16 22:35:25 jafo Exp $"

__doc__ = '''Module for dealing with credit card numbers.

This module implements several methods for dealing with credit card
numbers.  In particular, checking for legal numbers (based on the
check digit) and generating the check digit.

Example:

	import cccheck

	number = '21310000000000'
	number = number + cccheck.genMod10(number)
	print 'Generated: ', number
	print 'Verify: ', cccheck.verifyCardNumber(number)
'''

###################
def verifyMod10(s):
	'''Check a credit card number for validity using the mod10 algorithm.

	RETURNS: True if the card number is valid, false otherwise.
	ARGUMENTS:
		- s -- (String) Card number to verify.  Must only contain digits.
	'''
	double = 0
	sum = 0
	for i in range(len(s) - 1, -1, -1):
		for c in str((double + 1) * int(s[i])): sum = sum + int(c)
		double = (double + 1) % 2
	return((sum % 10) == 0)


################
def genMod10(s):
	'''Generate a mod-10 check digit.

	RETURNS: (String) Check digit.
	ARGUMENTS:
		- s -- (String) Card number to generate check from (should not include
				check digit).  Must only contain digits.
	'''
	double = 1
	sum = 0
	for i in range(len(s) - 1, -1, -1):
		for c in str((double + 1) * int(s[i])): sum = sum + int(c)
		double = (double + 1) % 2
	return(str(10 - (sum % 10)))


####################
def stripCardNum(s):
	'''Return card number with all non-digits stripped.

	RETURNS: (String) String consisting only of digits in input string.
	ARGUMENTS:
		- s -- (String) String to remove non-digits from.
	'''
	import re
	return(re.sub(r'[^0-9]', '', s))


########################
def verifyCardNumber(s):
	'''Return card type string if legal, None otherwise.
	Check the card number and return a string representing the card type if
	it could be a valid card number.

	RETURNS: (String) Credit card type string if legal.
			(None) if invalid.
	ARGUMENTS:
		- s -- (String) Card number to check.  Any non-digits are stripped out.
	'''
	s = stripCardNum(s)
	for name, prefix, length, algorithm in ccInfo:
		if len(s) == length and s[:len(prefix)] == prefix:
			if algorithm(s):
				return(name)
			break
	return(None)


##########################################
#  table with credit card type information
ccInfo = (
	#  type, prefix, length, verify algorithm
	( 'Visa', '4', 16, verifyMod10 ),
	( 'Visa', '4', 13, verifyMod10 ),
	( 'Mastercard', '51', 16, verifyMod10 ),
	( 'Mastercard', '52', 16, verifyMod10 ),
	( 'Mastercard', '53', 16, verifyMod10 ),
	( 'Mastercard', '54', 16, verifyMod10 ),
	( 'Mastercard', '55', 16, verifyMod10 ),
	( 'Discover', '6011', 16, verifyMod10 ),
	( 'American Express', '34', 15, verifyMod10 ),
	( 'American Express', '37', 15, verifyMod10 ),
	( 'Diners Club/Carte Blanche', '300', 14, verifyMod10 ),
	( 'Diners Club/Carte Blanche', '301', 14, verifyMod10 ),
	( 'Diners Club/Carte Blanche', '302', 14, verifyMod10 ),
	( 'Diners Club/Carte Blanche', '303', 14, verifyMod10 ),
	( 'Diners Club/Carte Blanche', '304', 14, verifyMod10 ),
	( 'Diners Club/Carte Blanche', '305', 14, verifyMod10 ),
	( 'Diners Club/Carte Blanche', '36', 14, verifyMod10 ),
	( 'Diners Club/Carte Blanche', '38', 14, verifyMod10 ),
	( 'JCB', '3', 16, verifyMod10 ),
	( 'JCB', '2131', 15, verifyMod10 ),
	( 'JCB', '1800', 15, verifyMod10 ),
	)

##########################
if __name__ == '__main__':
	number = '21310000000000'
	number = number + genMod10(number)
	print 'Generated: ', number
	print 'Verify: ', verifyCardNumber(number)
	print 'strip:', stripCardNum('4234-5678.9012/3456')
	print 'strip:', stripCardNum('4234 5678 9012 3456')
	print 'verify:', verifyCardNumber('4234 5678/9012-3456')
	print 'verify:', verifyCardNumber('5123456789012346')
	print 'verify:', verifyCardNumber('4992739871600')
	print 'verify:', verifyCardNumber('4992739871600008')
	print 'verify:', verifyCardNumber('6011000000000004')
	print 'verify:', verifyCardNumber('340000000000009')
	print 'verify:', verifyCardNumber('370000000000002')
	print 'verify:', verifyCardNumber('30000000000004')
	print 'verify:', verifyCardNumber('3100000000000003')



#!/usr/bin/env python

# Copyright (c) 2006-2008, David Allouche, Jp Calderone, Itamar Shtull-Trauring,
# Johan Dahlin, Olivier Grisel <olivier.grisel@ensta.org>
#
# All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""pyprof2calltree: profiling output which is readable by kcachegrind

This script can either take raw cProfile.Profile.getstats() log entries or
take a previously recorded instance of the pstats.Stats class.
"""

import cProfile
import pstats
import optparse
import os
import sys
import tempfile

__all__ = ['convert', 'visualize', 'CalltreeConverter']

class Code(object):
    pass

class Entry(object):
    pass

def pstats2entries(data):
    """Helper to convert serialized pstats back to a list of raw entries

    Converse opperation of cProfile.Profile.snapshot_stats()
    """
    entries = dict()
    allcallers = dict()

    # first pass over stats to build the list of entry instances
    for code_info, call_info in data.stats.items():
        # build a fake code object
        code = Code()
        code.co_filename, code.co_firstlineno, code.co_name = code_info

        # build a fake entry object
        cc, nc, tt, ct, callers = call_info
        entry = Entry()
        entry.code = code
        entry.callcount = cc
        entry.reccallcount = nc - cc
        entry.inlinetime = tt
        entry.totaltime = ct

        # to be filled during the second pass over stats
        entry.calls = list()

        # collect the new entry
        entries[code_info] = entry
        allcallers[code_info] = callers.items()

    # second pass of stats to plug callees into callers
    for entry in entries.itervalues():
        entry_label = cProfile.label(entry.code)
        entry_callers = allcallers.get(entry_label, [])
        for entry_caller, call_info in entry_callers:
            entries[entry_caller].calls.append((entry, call_info))

    return entries.values()

class CalltreeConverter(object):
    """Convert raw cProfile or pstats data to the calltree format"""

    kcachegrind_command = "kcachegrind %s"

    def __init__(self, profiling_data):
        if isinstance(profiling_data, basestring):
            # treat profiling_data as a filename of pstats serialized data
            self.entries = pstats2entries(pstats.Stats(profiling_data))
        elif isinstance(profiling_data, pstats.Stats):
            # convert pstats data to cProfile list of entries
            self.entries = pstats2entries(profiling_data)
        else:
            # assume this are direct cProfile entries
            self.entries = profiling_data
        self.out_file = None

    def output(self, out_file):
        """Write the converted entries to out_file"""
        self.out_file = out_file
        print >> out_file, 'events: Ticks'
        self._print_summary()
        for entry in self.entries:
            self._entry(entry)

    def visualize(self):
        """Launch kcachegrind on the converted entries

        kcachegrind must be present in the system path
        """

        if self.out_file is None:
            _, outfile = tempfile.mkstemp(".log", "pyprof2calltree")
            f = file(outfile, "wb")
            self.output(f)
            use_temp_file = True
        else:
            use_temp_file = False

        try:
            os.system(self.kcachegrind_command % self.out_file.name)
        finally:
            # clean the temporary file
            if use_temp_file:
                f.close()
                os.remove(outfile)
                self.out_file = None

    def _print_summary(self):
        max_cost = 0
        for entry in self.entries:
            totaltime = int(entry.totaltime * 1000)
            max_cost = max(max_cost, totaltime)
        print >> self.out_file, 'summary: %d' % (max_cost,)

    def _entry(self, entry):
        out_file = self.out_file

        code = entry.code
        #print >> out_file, 'ob=%s' % (code.co_filename,)

        co_filename, co_firstlineno, co_name = cProfile.label(code)
        print >> out_file, 'fi=%s' % (co_filename,)
        print >> out_file, 'fn=%s %s:%d' % (
            co_name, co_filename, co_firstlineno)

        inlinetime = int(entry.inlinetime * 1000)
        if isinstance(code, str):
            print >> out_file, '0 ', inlinetime
        else:
            print >> out_file, '%d %d' % (code.co_firstlineno, inlinetime)

        # recursive calls are counted in entry.calls
        if entry.calls:
            calls = entry.calls
        else:
            calls = []

        if isinstance(code, str):
            lineno = 0
        else:
            lineno = code.co_firstlineno

        for subentry, call_info in calls:
            self._subentry(lineno, subentry, call_info)
        print >> out_file

    def _subentry(self, lineno, subentry, call_info):
        out_file = self.out_file
        code = subentry.code
        #print >> out_file, 'cob=%s' % (code.co_filename,)
        co_filename, co_firstlineno, co_name = cProfile.label(code)
        print >> out_file, 'cfn=%s %s:%d' % (
            co_name, co_filename, co_firstlineno)
        print >> out_file, 'cfi=%s' % (co_filename,)
        print >> out_file, 'calls=%d %d' % (call_info[0], co_firstlineno)

        totaltime = int(call_info[3] * 1000)
        print >> out_file, '%d %d' % (lineno, totaltime)

def main():
    """Execute the converter using parameters provided on the command line"""

    usage = "%s [-k] [-o output_file_path] [-i input_file_path] [-r scriptfile [args]]"
    parser = optparse.OptionParser(usage=usage % sys.argv[0])
    parser.allow_interspersed_args = False
    parser.add_option('-o', '--outfile', dest="outfile",
                      help="Save calltree stats to <outfile>", default=None)
    parser.add_option('-i', '--infile', dest="infile",
                      help="Read python stats from <infile>", default=None)
    parser.add_option('-r', '--run-script', dest="script",
                      help="Name of the python script to run to collect"
                      " profiling data", default=None)
    parser.add_option('-k', '--kcachegrind', dest="kcachegrind",
                      help="Run the kcachegrind tool on the converted data",
                      action="store_true")
    options, args = parser.parse_args()


    outfile = options.outfile

    if options.script is not None:
        # collect profiling data by running the given script

        sys.argv[:] = [options.script] + args
        if not options.outfile:
            outfile = '%s.log' % os.path.basename(options.script)

        prof = cProfile.Profile()
        try:
            try:
                prof = prof.run('execfile(%r)' % (sys.argv[0],))
            except SystemExit:
                pass
        finally:
            kg = CalltreeConverter(prof.getstats())

    elif options.infile is not None:
        # use the profiling data from some input file
        if not options.outfile:
            outfile = '%s.log' % os.path.basename(options.infile)

        if options.infile == outfile:
            # prevent name collisions by appending another extension
            outfile += ".log"

        kg = CalltreeConverter(pstats.Stats(options.infile))

    else:
        # at least an input file or a script to run is required
        parser.print_usage()
        sys.exit(2)

    if options.outfile is not None or not options.kcachegrind:
        # user either explicitely required output file or requested by not
        # explicitely asking to launch kcachegrind
        print "writing converted data to: " + outfile
        kg.output(file(outfile, 'wb'))

    if options.kcachegrind:
        print "launching kcachegrind"
        kg.visualize()


def visualize(profiling_data):
    """launch the kcachegrind on `profiling_data`

    `profiling_data` can either be:
        - a pstats.Stats instance
        - the filename of a pstats.Stats dump
        - the result of a call to cProfile.Profile.getstats()
    """
    converter = CalltreeConverter(profiling_data)
    converter.visualize()

def convert(profiling_data, outputfile):
    """convert `profiling_data` to calltree format and dump it to `outputfile`

    `profiling_data` can either be:
        - a pstats.Stats instance
        - the filename of a pstats.Stats dump
        - the result of a call to cProfile.Profile.getstats()

    `outputfile` can either be:
        - a file() instance open in write mode
        - a filename
    """
    converter = CalltreeConverter(profiling_data)
    if isinstance(outputfile, basestring):
        f = file(outputfile, "wb")
        try:
            converter.output(f)
        finally:
            f.close()
    else:
        converter.output(outputfile)


if __name__ == '__main__':
    sys.exit(main())



"""
Transactions of attributes by inheriting the Transaction class

Basic Usage:

    >>> class Test(Transaction):
    ...     pass        
    >>> a = Test()
    >>> a.test = "old state"
    >>> a.commit()
    >>> a.test = "bad state, roll me back"
    >>> a.rollback()
    >>> assert(a.test == "old state")
    
See also: http://www.harald-hoyer.de/linux/pythontransactionclass
"""

__author__ = "Harald Hoyer <harald@redhat.com>"


import copy

def _checksetseen(what, seen):
    "checks and sets the obj id in seen"
    if what in seen:
        return True
    seen.add(what)
    return False

class Transaction(object):
    """
    This class allows sub-classes to commit changes to an instance to a 
    history, and rollback to previous states.
        
    Because the class only stores attributes in self.__dict__ sub-classes
    need to use the methods __getstate__ and __setstate__ to provide additional
    state information. See the Transactionlist below for an example usage.    
    """

    def commit(self, **kwargs):
        """
        Commit the object state.
        
        If the optional argument "deep" is set to False,
        objects of class Transaction stored in this object will
        not be committed.
        """
        seen = kwargs.get("_commit_seen", set())
        if _checksetseen(id(self), seen): 
            return
        deep = kwargs.get("deep", True)
        
        # Do not deepcopy the Transaction objects. We want to keep the 
        # reference. Instead commit() them.     
        state = dict()
        for key, val in self.__dict__.items():
            if isinstance(val, Transaction):
                state[key] = val
                if deep:
                    val.commit(_commit_seen = seen)
            elif key == "__l":
                # do not deepcopy our old state
                state[key] = val
            else:
                state[key] = copy.deepcopy(val)
                
        if hasattr(self, '__getstate__'):            
            state = (state, getattr(self, '__getstate__')())

        self.__dict__["__l"] = state
                
    def rollback(self, **kwargs):
        """
        Rollback the last committed object state.
        
        If the optional argument "deep" is set to False,
        objects of class Transaction stored in this object will
        not be rolled back.
        """
        seen = kwargs.get("_rollback_seen", set())
        if _checksetseen(id(self), seen):
            return
        
        deep = kwargs.get("deep", True)
        state = None
        extrastate = None
        gotstate = False
        gotextrastate = False
        if "__l" in self.__dict__:
            state = self.__dict__["__l"]
            gotstate = True
            if type(state) is tuple:
                gotextrastate = True
                (state, extrastate) = state

        # rollback our childs, then ourselves
        for child in self.__dict__.values():
            if isinstance(child, Transaction):
                if deep:
                    child.rollback(_rollback_seen = seen)
 
        if gotstate:
            self.__dict__.clear()
            self.__dict__.update(state)
            
        if gotextrastate and hasattr(self, '__setstate__'):
            getattr(self, '__setstate__')(extrastate)


class Transactionlist(list, Transaction):
    """
    An example subclass of list, which inherits transactions.
    
    Due to the special list implementation, we need the 
    __getstate__ and __setstate__ methods.
    
    See the code for the implementation.
    """
    def commit(self, **kwargs):
        """
        Commit the object state.
        
        If the optional argument "deep" is set to False,
        objects of class Transaction stored in this object will
        not be committed.
        """
        # make a local copy of the recursive marker
        seen = set(kwargs.get("_commit_seen", set()))
        
        super(Transactionlist, self).commit(**kwargs)

        if _checksetseen(id(self), seen): 
            return
        
        deep = kwargs.get("deep", True)
        if deep:
            for val in self:
                if isinstance(val, Transaction):
                    val.commit()
        
    def rollback(self, **kwargs):
        """
        Rollback the last committed object state.
        
        If the optional argument "deep" is set to False,
        objects of class Transaction stored in this object will
        not be rolled back.
        """
        # make a local copy of the recursive marker
        seen = set(kwargs.get("_rollback_seen", set()))

        super(Transactionlist, self).rollback(**kwargs)

        if _checksetseen(id(self), seen):
            return
        
        deep = kwargs.get("deep", True)
        if deep:
            for val in self:
                if isinstance(val, Transaction):
                    val.rollback()

                
    def __getstate__(self):
        """
        return a deepcopy of all non Transaction class objects in our list, 
        and a reference for the committed Transaction objects.
        
        """
        state  = []
        for val in self:
            if isinstance(val, Transaction):
                state.append(val)
            else:
                state.append(copy.deepcopy(val))
                
        return state        

    def __setstate__(self, state):
        "clear the list and restore all objects from the state"
        del self[:]
        self.extend(state)




prepositions = """
aboard
about
above
absent
across
after
against
along
alongside
amid
amidst
among
amongst
around
as
astride
at
atop
barring
before
behind
below
beneath
beside
besides
between
beyond
but
by
despite
down
during
except
failing
following
for
from
in
inside
into
like
mid
minus
near
next
notwithstanding
of
off
on
onto
opposite
outside
over
past
plus
regarding
round
save
since
than
through
throughout
till
times
to
toward
towards
under
underneath
unlike
until
up
upon
via
with
within
without
according to
ahead of
as to
aside from
because of
close to
due to
far from
in to
inside of
instead of
near to
next to
on to
out of
outside of
owing to
prior to
subsequent to
as far as
as well as
by means of
in accordance with
in addition to
in front of
in place of
in spite of
on account of
on behalf of
on top of
with regard to
in case of
"""



from email import encoders
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import mimetypes
import os
import re
import smtplib

class Email:
    """
    This class handles the creation and sending of email messages
    via SMTP.  This class also handles attachments and can send
    HTML messages.  The code comes from various places around
    the net and from my own brain.
    """
    def __init__(self, smtpServer):
        """
        Create a new empty email message object.

        @param smtpServer: The address of the SMTP server
        @type smtpServer: String
        """
        self._textBody = None
        self._htmlBody = None
        self._subject = ""
        self._smtpServer = smtpServer
        self._reEmail = re.compile("^([\\w \\._]+\\<[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\\>|[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?)$")
        self.clearRecipients()
        self.clearAttachments()
    
    def send(self):
        """
        Send the email message represented by this object.
        """
        # Validate message
        if self._textBody is None and self._htmlBody is None:
            raise Exception("Error! Must specify at least one body type (HTML or Text)")
        if len(self._to) == 0:
            raise Exception("Must specify at least one recipient")
        
        # Create the message part
        if self._textBody is not None and self._htmlBody is None:
            msg = MIMEText(self._textBody, "plain")
        elif self._textBody is None and self._htmlBody is not None:
            msg = MIMEText(self._htmlBody, "html")
        else:
            msg = MIMEMultipart("alternative")
            msg.attach(MIMEText(self._textBody, "plain"))
            msg.attach(MIMEText(self._htmlBody, "html"))
        # Add attachments, if any
        if len(self._attach) != 0:
            tmpmsg = msg
            msg = MIMEMultipart()
            msg.attach(tmpmsg)
        for fname,attachname in self._attach:
            if not os.path.exists(fname):
                print "File '%s' does not exist.  Not attaching to email." % fname
                continue
            if not os.path.isfile(fname):
                print "Attachment '%s' is not a file.  Not attaching to email." % fname
                continue
            # Guess at encoding type
            ctype, encoding = mimetypes.guess_type(fname)
            if ctype is None or encoding is not None:
                # No guess could be made so use a binary type.
                ctype = 'application/octet-stream'
            maintype, subtype = ctype.split('/', 1)
            if maintype == 'text':
                fp = open(fname)
                attach = MIMEText(fp.read(), _subtype=subtype)
                fp.close()
            elif maintype == 'image':
                fp = open(fname, 'rb')
                attach = MIMEImage(fp.read(), _subtype=subtype)
                fp.close()
            elif maintype == 'audio':
                fp = open(fname, 'rb')
                attach = MIMEAudio(fp.read(), _subtype=subtype)
                fp.close()
            else:
                fp = open(fname, 'rb')
                attach = MIMEBase(maintype, subtype)
                attach.set_payload(fp.read())
                fp.close()
                # Encode the payload using Base64
                encoders.encode_base64(attach)
            # Set the filename parameter
            if attachname is None:
                filename = os.path.basename(fname)
            else:
                filename = attachname
            attach.add_header('Content-Disposition', 'attachment', filename=filename)
            msg.attach(attach)
        # Some header stuff
        msg['Subject'] = self._subject
        msg['From'] = self._from
        msg['To'] = ", ".join(self._to)
        msg.preamble = "You need a MIME enabled mail reader to see this message"
        # Send message
        msg = msg.as_string()
        server = smtplib.SMTP(self._smtpServer)
        server.sendmail(self._from, self._to, msg)
        server.quit()
    
    def setSubject(self, subject):
        """
        Set the subject of the email message.
        """
        self._subject = subject
    
    def setFrom(self, address):
        """
        Set the email sender.
        """
        if not self.validateEmailAddress(address):
            raise Exception("Invalid email address '%s'" % address)
        self._from = address
    
    def clearRecipients(self):
        """
        Remove all currently defined recipients for
        the email message.
        """
        self._to = []
    
    def addRecipient(self, address):
        """
        Add a new recipient to the email message.
        """
        if not self.validateEmailAddress(address):
            raise Exception("Invalid email address '%s'" % address)
        self._to.append(address)
    
    def setTextBody(self, body):
        """
        Set the plain text body of the email message.
        """
        self._textBody = body
    
    def setHtmlBody(self, body):
        """
        Set the HTML portion of the email message.
        """
        self._htmlBody = body
    
    def clearAttachments(self):
        """
        Remove all file attachments.
        """
        self._attach = []
    
    def addAttachment(self, fname, attachname=None):
        """
        Add a file attachment to this email message.

        @param fname: The full path and file name of the file
                      to attach.
        @type fname: String
        @param attachname: This will be the name of the file in
                           the email message if set.  If not set
                           then the filename will be taken from
                           the fname parameter above.
        @type attachname: String
        """
        if fname is None:
            return
        self._attach.append( (fname, attachname) )
    
    def validateEmailAddress(self, address):
        """
        Validate the specified email address.
        
        @return: True if valid, False otherwise
        @rtype: Boolean
        """
        if self._reEmail.search(address) is None:
            return False
        return True
    
if __name__ == "__main__":
    # Run some tests
    mFrom = "Test User <test@mydomain.com>"
    mTo = "you@yourdomain.com"
    m = Email("mail.mydomain.com")
    m.setFrom(mFrom)
    m.addRecipient(mTo)
    
    # Simple Plain Text Email
    m.setSubject("Plain text email")
    m.setTextBody("This is a plain text email <b>I should not be bold</b>")
    m.send()
    
    # Plain text + attachment
    m.setSubject("Text plus attachment")
    m.addAttachment("/home/user/image.png")
    m.send()

    # Simple HTML Email
    m.clearAttachments()
    m.setSubject("HTML Email")
    m.setTextBody(None)
    m.setHtmlBody("The following should be <b>bold</b>")
    m.send()

    # HTML + attachment
    m.setSubject("HTML plus attachment")
    m.addAttachment("/home/user/image.png")
    m.send()
    
    # Text + HTML
    m.clearAttachments()
    m.setSubject("Text and HTML Message")
    m.setTextBody("You should not see this text in a MIME aware reader")
    m.send()
    
    # Text + HTML + attachment
    m.setSubject("HTML + Text + attachment")
    m.addAttachment("/home/user/image.png")
    m.send()





## IDEA: Maybe we can extend the awesome contextlib.contextmanager decorator:
## 
## from contextlib import GeneratorContextManager
## 
## This is a broken first attempt:
## def contextmanager2(f):
## 
##     @wraps(f)
##     def helper(decorated_f=None):
## 
##         class xx(GeneratorContextManager):
##             def __call__(self, *args, **kw):
##                 with self:
##                     return decorated_f(*args, **kw)
## 
##         return xx(f())
## 
##     return helper
## 
## @contextmanager2
## def preserve_cwd():
##     """
##     context-manager which doubles as a decorator that preserve current
##     working directory.
## 
##     Usage example:
## 
##     As a decorator:
##         >>> before = os.getcwd()
##         >>> @preserve_cwd
##         ... def foo():
##         ...     os.chdir('..')
##         >>> before == os.getcwd()
##         True
## 
##     As a context-manager:
##         >>> before = os.getcwd()
##         >>> with preserve_cwd():
##         ...     os.chdir('..')
##         >>> before == os.getcwd()
##         True
##     """
##     cwd = os.getcwd()
##     yield
##     os.chdir(cwd)


__all__ = ['ContextDecorator']

from functools import wraps

class ContextDecorator(object):
    """
    Create objects that act as both context managers *and* as decorators, and
    behave the same in both cases.

    This implementation of ContextDecorator is based on Michael Foord's
    http://pypi.python.org/pypi/contextdecorator.

    Original copyright / license:
      # Copyright (C) 2007-2010 Michael Foord
      # E-mail: michael AT voidspace DOT org DOT uk
      # http://pypi.python.org/pypi/contextdecorator
    """

    def before(self):
        """
        Called on entering the with block or starting the decorated function.

        If used in a with statement whatever this method returns will be the
        context manager.
        """
        return self

    def after(self, exc_type, exc_value, traceback):
        """
        Called on exit. Arguments and return value of this method have
        the same meaning as the __exit__ method of a normal context
        manager.

        Note: If the after method of was triggered by an exception, and we wish
        to prevent the exception from being propagated, this method it should
        return a true value. Otherwise, the exception will be processed
        normally (do not reraise the exception here!).
        """
        return False

    def __call__(self, f):
        @wraps(f)
        def inner(*args, **kw):
            with self:
                return f(*args, **kw)
        return inner

    def __enter__(self):
        return self.before()

    def __exit__(self, exc_type, exc_value, traceback):
        return self.after(exc_type, exc_value, traceback)



#______________________________________________________________________________
# Unit tests

import unittest

class mycontext(ContextDecorator):
    started = False
    exc = None
    catch = False

    def before(self):
        self.started = True
        return self

    def after(self, exc_type, exc_val, tb):
        self.exc = exc_type
        return self.catch

class TestContext(unittest.TestCase):

    def test_context(self):
        context = mycontext()
        self.assertFalse(context.started)

        with context as result:
            self.assertTrue(result is context)
            self.assertTrue(context.started)

        with mycontext() as result:
            self.assertTrue(isinstance(context, mycontext))
            self.assertTrue(context.started)

    def test_exceptions(self):

        class MyException(Exception): pass

        @mycontext()
        def test():
            raise MyException()

        # assert raises MyException
        try:
            with mycontext():
                raise MyException()
        except MyException:
            pass
        else:
            self.assertTrue(False)

        # assert raises MyException
        try:
            test()
        except MyException:
            pass
        else:
            self.assertTrue(False)

    def test_decorator(self):

        @mycontext()
        def test(a,b,c):
            "my doc string"
            return (a,b,c)

        A,B,C = 'A','B','C'
        a,b,c = test(A,B,C)
        assert a is A and b is B and c is C
        assert test.__name__ == 'test'
        assert test.__doc__ == 'my doc string'

    def test_decorating_method(self):

        class Test(object):

            def __init__(self):
                self.a, self.b, self.c = "ABC"

            @mycontext()
            def method(self, a, b, c=None):
                self.a = a
                self.b = b
                self.c = c

        test = Test()
        test.method(1, 2)
        self.assertEqual(test.a, 1)
        self.assertEqual(test.b, 2)
        self.assertEqual(test.c, None)

        test.method('a', 'b', 'c')
        self.assertEqual(test.a, 'a')
        self.assertEqual(test.b, 'b')
        self.assertEqual(test.c, 'c')


if __name__ == '__main__':
    unittest.main()
    import doctest
    doctest.testmod()



import re
from sys import stdout, stderr, stdin
from glob import glob
from cStringIO import StringIO

def print_parse(t, out=stdout.write):
    "Print parse formatted as an s-expression."
    def pp(t):
        if isinstance(t, basestring):                    # base case
            return out(t)
        if len(t) == 1:
            if t[0]:
                pp(t[0])
            return
        label, children = t[0], t[1:]
        assert isinstance(label, basestring)
        out('(%s ' % label)
        n = len(children)
        for i, child in enumerate(children):
            pp(child)   # first child already indented
            if i != n-1:                                 # no space after last child
                out(' ')
        out(')')
    pp(t)
    out('\n')


def pprint(t, out=stdout.write):
    "Pretty print tree as a tabbified s-expression."
    def pp(t, indent=0, indentme=True):
        if indentme:
            out(' '*indent)
        if isinstance(t, basestring):                    # base case
            return out(t)
        if len(t) == 1:
            if t[0]:
                pp(t[0], indent, indentme)
            return
        label, children = t[0], t[1:]
        assert isinstance(label, basestring)
        out('(%s ' % label)
        n = len(children)
        for i, child in enumerate(children):
            pp(child, indent + len(label) + 2, i != 0)   # first child already indented
            if i != n-1:                                 # no newline after last child
                out('\n')
        out(')')
    pp(t)
    out('\n')


def pformat(t):
    "Pretty print tree as a tabbified s-expression."
    y = StringIO()
    pprint(t, out=y)
    return y.getvalue()


def sexpr(s, add_root=True):
    """

    Example usage:

      >>> sexpr('(S (NP Papa) (VP (V ate) (NP (Det the) (N caviar))))')
      ['S', ['NP', 'Papa'], ['VP', ['V', 'ate'], ['NP', ['Det', 'the'], ['N', 'caviar']]]]


    TO match the Penn tree bank we add a ROOT symbol in the following case

      >>> sexpr('((S (NP Papa) (VP (V ate) (NP (Det the) (N caviar)))))')
      ['ROOT', ['S', ['NP', 'Papa'], ['VP', ['V', 'ate'], ['NP', ['Det', 'the'], ['N', 'caviar']]]]]

    """


    s = s[s.find('('):]
    tree = []
    stack = []  # top of stack (index -1) points to current node in tree
    stack.append(tree)
    curtok = ""
    depth = 0
    for c in s:
        if c=='(':
            new = []
            stack[-1].append(new)
            stack.append(new)
            curtok = ""
            depth += 1
        elif c==')':
            if curtok:
                stack[-1].append(curtok)
                curtok = ""
            stack.pop()
            curtok = ""
            depth -= 1
        #elif c.isspace():
        elif c in (' ','\t','\r','\n'):  ## dont want funny unicode ones?
            if curtok:
                stack[-1].append(curtok)
                curtok = ""
        else:
            curtok += c
        if depth < 0:
            raise BadSexpr("Too many closing parens")
    if depth > 0:
        raise BadSexpr("Didn't close all parens, depth %d" % depth)
    root = tree[0]
    # weird, treebank parses have an extra, unlabeled node on top
    if isinstance(root[0], list) and add_root:
        root = ["ROOT"] + root
    return root

class BadSexpr(Exception):
    pass



#def features(t):
#    print 'S, NP, VP'


def main():

    for filename in glob('/home/timv/projects/ldp/data/LDC99T42/treebank_3/parsed/mrg/wsj/*/*.mrg'):
        #print >> stderr, filename
        print >> stderr, '.',

        with file(filename) as f:
            contents = f.read()
            chunks = contents.split('( (')

        # parse file using sexpr
        for chunk in chunks:

            if not chunk.strip():
                continue

            chunk = '( (' + chunk

            print chunk

            try:
                tree = sexpr(chunk)
            except BadSexpr:
                print >> stderr
                print >> stderr, 'failed to parse tree'
                print >> stderr, chunk
                continue

            print tree
#            features(tree)



if __name__ == '__main__':
    main()



#!/usr/bin/python

# Based on a recipe 
# http://code.activestate.com/recipes/577547-flatten-xml-to-xpath-syntax-lines/

"""
Transforms a XML file into a flat text output, with XPath-like syntax, one line
per XML node or attribute. This format is more suitable for working with
standard unix command-line utils (sed, grep, ... etc).
"""


from xml.etree.ElementTree import parse

def linearize(el, path, numbered=True):

    # print text value if not empty
    text = (el.text or '').strip()
    if not text:
        yield path, ''
    else:
        # Several lines?
        lines = text.splitlines()
        if len(lines) > 1:
            for lineno, line in enumerate(lines):
                yield path + "[line %d]" % (lineno + 1), line
        else:
            yield path, text

    # Print attributes
    for name, val in el.items():
        yield path + "/@" + name, val

    # Counter on the sibling element names
    counter = {}
    for child in el:
        tag = child.tag
        if numbered:
            if tag in counter:     # number tag names which have already occurred
                counter[tag] += 1
                tag = "%s[%d]" % (tag, counter[tag])
            else:
                counter[tag] = 1

        # recurse
        for x in linearize(child, path + '/' + tag):
            yield x

def linearize_stream(stream, prefix='', numbered=True):
    tree = parse(stream)
    root = tree.getroot()
    for x in linearize(root, prefix + "//" + root.tag, numbered=numbered):
        yield x

def process(stream, prefix=''):
    for path, content in linearize_stream(stream, prefix):
        print '%s\t%s' % (path, content)


def main():
    import sys
    if len(sys.argv) > 2:
        print 'sorry one file at a time.'
        sys.exit(1)
    if len(sys.argv) == 2:
        with file(sys.argv[1]) as f:
            process(f)
    else:
        process(sys.stdin)

if __name__ == '__main__':
    main()




import warnings

_NOTFOUND = object()


class Flipdict(dict):
    """An injective (one-to-one) python dict.  Ensures that each key
    maps to a unique value, and each value maps back to that same key.
    Each instance has a "flip" attribute to access the inverse
    mapping.

    A Flipdict is a dict, implementing the dict API (except dict.fromkeys)::
        >>> keys = (1, 2, 3)
        >>> vals = ('one', 'two', 'three')
        >>> fd = Flipdict(zip(keys, vals))
        >>> fd
        Flipdict({1: 'one', 2: 'two', 3: 'three'})
        >>> fd[1]
        'one'
        >>> fd.get(4, 'four')
        'four'

    Each Flipdict has a "flip" attribute, a Flipdict with the inverse mapping::
        >>> fd is fd.flip.flip
        True
        >>> fd.flip['one']
        1
        >>> fd.flip.setdefault('four', 4)
        4
        >>> fd
        Flipdict({1: 'one', 2: 'two', 3: 'three', 4: 'four'})
        >>> fd.flip
        Flipdict({'four': 4, 'one': 1, 'three': 3, 'two': 2})

    NOTE: Remapping a key -- OR A VALUE -- will erase the old mapping.
    This behavior maintains the inverse mapping, but may be
    unexpected.  Check the docstring of Flipdict.__setitem__ for more
    details.  An example::
        >>> fd.flip['one'] = 4  # Erases both (1,'one') and (4,'four') !!
        >>> fd
        Flipdict({2: 'two', 3: 'three', 4: 'one'})
    
    Some other misc. examples::
        >>> del fd.flip.flip.flip['two']
        >>> fd.flip.update({'zero':0, 'zilch':0, 'zip':0}, naught=0)
        >>> fd  # We only expect a single mapping for key=0.
        Flipdict({0: 'naught', 3: 'three', 4: 'one'})
        >>> sorted(fd.flip.items())
        [('naught', 0), ('one', 4), ('three', 3)]
    """

    def __init__(self, *args, **kw):
        """Similar to dict.__init__

        Examples::
            >>> keys = (1, 2, 3)
            >>> vals = ('one', 'two', 'three')
            >>> fd = Flipdict(zip(keys, vals))
            >>> fd == Flipdict({1: 'one', 2: 'two', 3: 'three'})
            True
            >>> fd == Flipdict(one=1, two=2, three=3).flip
            True

        NOTE: If multiple keys map to the same value, then only ONE of
        those (key, value) mappings will be present after the __init__.
        The other (key, value) pairs will have vanished::
            >>> d = dict.fromkeys(range(5), 'value')
            >>> len( Flipdict(d) )
            1
        """
        self._flip = dict.__new__(self.__class__)
        setattr(self._flip, "_flip", self)
        for key, val in dict(*args, **kw).iteritems():
            self[key] = val

    @property
    def flip(self):
        """The inverse mapping.

        Example::
            >>> keys = (1, 2, 3)
            >>> vals = ('one', 'two', 'three')
            >>> fd = Flipdict(zip(keys, vals))
            >>> fd.flip['two']
            2
            >>> fd.flip.flip[2]
            'two'
            >>> fd is fd.flip.flip
            True
            >>> fd.flip is fd.flip.flip.flip
            True
        """
        return self._flip


    #{ Non-mutating methods that are NOT delegated to the dict superclass.

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, dict(self))

    __str__ = __repr__

    def copy(self):
        """Cf. dict.copy"""
        return self.__class__(self)

    @classmethod
    def fromkeys(cls, keys, value=None):
        """Cf. dict.fromkeys

        NOTE: This method is not useful for a Flipdict, because each
        key must map to a UNIQUE value.
        """
        warnings.warn('%s.fromkeys is not useful'
                      ' --- each key must map to a UNIQUE value' % cls.__name__,
                      DeprecationWarning)
        return cls(dict.fromkeys(keys, value))

    #}


    #{ Mutating methods.  These must keep the inverse mapping in sync!

    def __setitem__(self, key, val):
        """Similar to dict.__setitem__

        NOTE: Remapping a key -- OR A VALUE -- will erase the old
        mapping.  This behavior maintains the inverse mapping, but may
        be unexpected::        
            >>> keys = (1, 2, 3)
            >>> vals = ('ant', 'bug', 'cat')
            >>> fd = Flipdict(zip(keys, vals))
            >>> fd
            Flipdict({1: 'ant', 2: 'bug', 3: 'cat'})

            >>> fd.setdefault(4, 'dog')  # Both key and value are new.
            'dog'
            >>> fd[1] = 'ant'            # This mapping already exists.
            >>> fd[1] = 'ape'            # Map an old key to a new value.
            >>> fd                       # No surprises; behaves like a dict.
            Flipdict({1: 'ape', 2: 'bug', 3: 'cat', 4: 'dog'})

            >>> fd['ant'] = 'bug'        # AHA! Map a new key to an old value!
            >>> fd[2]
            Traceback (most recent call last):
                ....
            KeyError: 2
            >>> fd
            Flipdict({'ant': 'bug', 1: 'ape', 3: 'cat', 4: 'dog'})

            >>> oldKey = fd[4]
            >>> oldVal = fd.flip['cat']
            >>> fd.flip['cat'] = 4       # AHA! Re-map with old key AND value!
            >>> oldKey in fd
            False
            >>> oldVal in fd.flip
            False
            >>> fd
            Flipdict({'ant': 'bug', 1: 'ape', 4: 'cat'})
        """
        v = self.get(key, _NOTFOUND)
        k = self._flip.get(val, _NOTFOUND)
        if k is not _NOTFOUND:  dict.__delitem__(self, k)
        if v is not _NOTFOUND:  dict.__delitem__(self._flip, v)
        dict.__setitem__(self,       key, val)
        dict.__setitem__(self._flip, val, key)

    def setdefault(self, key, default = None):
        """Cf. dict.setdefault

        NOTE: Remapping a key -- OR A VALUE -- will erase the old
        mapping.  This behavior maintains the inverse mapping, but may
        be unexpected.  Cf. Flipdict.__setitem__.
        """
        # Copied from python's UserDict.DictMixin code.
        try:
            return self[key]
        except KeyError:
            self[key] = default
            return default

    def update(self, other = None, **kwargs):
        """Cf. dict.update

        NOTE: Remapping a key -- OR A VALUE -- will erase the old
        mapping.  This behavior maintains the inverse mapping, but may
        be unexpected.  Cf. Flipdict.__setitem__.
        """
        # Copied from python's UserDict.DictMixin code.
        # Make progressively weaker assumptions about "other"
        if other is None:
            pass
        elif hasattr(other, 'iteritems'):  # iteritems saves memory and lookups
            for k, v in other.iteritems():
                self[k] = v
        elif hasattr(other, 'keys'):
            for k in other.keys():
                self[k] = other[k]
        else:
            for k, v in other:
                self[k] = v
        if kwargs:
            self.update(kwargs)

    def __delitem__(self, key):
        val = dict.pop(self, key)
        dict.__delitem__(self._flip, val)

    def pop(self, key, *args):
        """Cf. dict.pop"""
        val = dict.pop(self, key, *args)
        dict.__delitem__(self._flip, val)
        return val

    def popitem(self):
        """Cf. dict.popitem"""
        key, val = dict.popitem(self)
        dict.__delitem__(self._flip, val)
        return key, val

    def clear(self):
        """Cf. dict.clear"""
        dict.clear(self)
        dict.clear(self._flip)


if __name__=='__main__':
    import doctest
    doctest.testmod(optionflags=doctest.ELLIPSIS, verbose=True)



from __future__ import division
from numpy import array, empty, zeros
import math
from math import exp as math_exp, log as math_log

inf = float('infinity')
ninf = float('-infinity')

def exp(x):
    try:
        return math_exp(x)
    except OverflowError:
        return inf

def log(x):
    if x < 1e-10:
        return 0.0
    else:
        return math_log(x)

log_of_2 = math.log(2)

def entropy(p):
    """Calculate entropy of a discrete random variable with given probabilities."""
    return -sum(pi*log(pi) for pi in p if pi > 0) / log_of_2

def kl_divergence(p, q):
    """ Compute KL divergence of two vectors, K(p || q).
    NOTE: If any value in q is 0.0 then the KL-divergence is infinite.
    """
    assert len(p) == len(q)
    kl = sum(p[i] * log(p[i] / q[i]) for i in xrange(len(p)) if p[i] != 0.0)
    return kl / log_of_2

def jensen_shannon_divergence(p, q):
    """ Returns the Jensen-Shannon divergence. """
    assert len(p) == len(q)
    average = zeros(len(p))
    for i in xrange(len(p)):
        average[i] += (p[i] + q[i]) / 2.0
    return (kl_divergence(p, average) + kl_divergence(q, average)) / 2.0

def normalize(x):
    """ Divide each element of the array by the sum of the elements. """
    return (1.0/sum(x)) * array(x)

def normalize_inplace(x):
    """ Divide each element of the array by the sum of the elements IN PLACE. """
    if isinstance(x,dict):
        Z = sum(x.itervalues())
        for k in x:
            x[k] /= Z
    else:
        Z = sum(x)
        for i in xrange(len(x)):
            x[i] /= Z

def exp_normalize(A):
    """
    Exponentiate elements of the array, and then normalize them to sum to one.

    exp(xi) / (exp(x1) + ... + exp(xn))
         = exp(xi-B)*exp(B) / (exp(B)*(exp(x1-B) + ... + exp(xn-B)))  for any B
         = exp(xi-B) / (exp(x1-B) + ... + exp(xn-B))                  for any B
    """
    M = max(A)
    B = empty(len(A))
    for k in xrange(len(A)):
        B[k] = exp(A[k] - M)
    Z = sum(B)
    for k in xrange(len(A)):
        B[k] /= Z
    return B

def exp_normalize_inplace(A):
    """ Exponentiate the elements of the array, and then normalize them to sum to one. """
    if isinstance(A,dict):
        M = max(A.itervalues())
        for k in A:
            A[k] = exp(A[k] - M)
        Z = sum(A.itervalues())
        for k in A:
            A[k] /= Z
    else:
        M = max(A)
        for k in xrange(len(A)):
            A[k] = exp(A[k] - M)
        Z = sum(A)
        for k in xrange(len(A)):
            A[k] /= Z

def normalize_log_prob(a):
    # normalizeLogProb: [log(a), log(b), log(c)] --> [log(a/Z), log(b/Z), log(c/Z)] where Z = a+b+c
    # expNormalize: [log(a), log(b), log(c)] --> [a/Z, b/Z, c/Z] where Z=a+b+c
    b = exp_normalize(a)
    for i in xrange(len(b)):
        b[i] = log(b[i])
    return b

def normalize_log_prob_inplace(a):
    exp_normalize_inplace(a)
    for i in xrange(len(a)):
        a[i] = log(a[i])

def logsumexp(x):
    """
    The log-sum-exp trick:
      log(sum(exp(xi) for xi in X)) <=> log(sum(exp(xi-B) for xi in X)) + B   for any B

    picking B = max(X) ensures that the largest value you exponentiate will be
    zero. So there definitely won't be overflow. Even if the rest underflow
    you'll still get a reasonable value.
    """
    B = max(x)
    return log(sum(exp(x-B) for x in x if x > ninf)) + B

def sum_two_log_probs(a, b):
    """
    Returns the sum of two doubles expressed in log space
       sumLogProb = log (e^a + e^b)
                  = log e^a(1 + e^(b-a))
                  = a + log (1 + e^(b-a))

    By exponentiating (b-a), we obtain better numerical precision than
    we would if we calculated e^a or e^b directly.
    """
    if b < a:
        return a + log(1 + exp(b-a))
    else:
        return b + log(1 + exp(a-b))

def subtract_log_prob(a, b):
    """ Returns the difference of two doubles expressed in log space """
    if b < a:
        return a + log(1 - exp(b-a))
    else:
        return b + log(1 - exp(a-b))


if __name__ == '__main__':
    import doctest; doctest.testmod()

    assert entropy((0.5, 0.5)) == 1.0
    assert abs(entropy((0.75, 0.25)) - 0.8112781244) < 1e-10
    assert abs(entropy((0.1, 0.1, 0.8)) == 0.9219280948) < 1e-10



conjunctions = """
for
and
nor
but
or
yet
so
after
because
although
if
before
since
though
unless
when
now that
even though
only if
while
as
whereas
whether or not
since
in order that
while
even if
until
so
in case
in case that
"""



import sys
from time import time, sleep

from operator import getitem, sub, mul
from itertools import *
from random import shuffle
from collections import defaultdict, deque
from operator import itemgetter

# IDEAS:
# * progress_meter: updates based on how much work was dones, e.g.,
#     >> p = progress_meter(100)
#     0.0%
#     >> p.update(10)
#     10.0%

def argmax(f, seq):
    """
    >>> argmax(lambda x: -x**2 + 1, range(-10,10))
    0
    """
    return argmax2(f,seq)[1]

def argmax2(f, seq):
    """
    >>> argmax2(lambda x: -x**2 + 1, range(-10,10))
    (1, 0)
    """
    return max(((f(x),x) for x in seq), key=itemgetter(0))

def argmin(f, seq):
    return argmin2(f,seq)[1]

def argmin2(f, seq):
    return min(((f(x),x) for x in seq), key=itemgetter(0))

def groupby2(s, key=lambda x: x):
    """
    Eager version of groupby which does what you'd expect groupby to do.

    >>> groupby2(range(10), lambda x: x % 2)
    {0: [0, 2, 4, 6, 8], 1: [1, 3, 5, 7, 9]}
    """
    groups = defaultdict(list)
    for x in s:
        groups[key(x)].append(x)
    return dict(groups)

def atmost(k, seq):
    """
    >>> atmost(1, [0,0,0])
    True

    >>> atmost(1, [0,1,0])
    True

    >>> atmost(1, [0,1,1])
    False
    """
    count = 0
    if count > k:
        return False
    for x in seq:
        if x:
            count += 1
            if count > k:
                return False
    return True

def partition(data, proportion):
    """
    Deterministically partition `data` according to proportion.
    Note: assumes sum(proportion) <= 1.0

    >>> partition(range(10), [0.3, 0.7])
    [[0, 1, 2], [3, 4, 5, 6, 7, 8, 9]]
    """
    assert sum(proportion) <= 1
    data = list(data)
    D = iter(data)
    N = len(data)
    return [list(take(j, D)) for j in [int(p*N) for p in proportion]]


def breadth_first(tree, children=iter, depth=-1, queue=None):
    """Traverse the nodes of a tree in breadth-first order.
    (No need to check for cycles.)
    The first argument should be the tree root;
    children should be a function taking as argument a tree node
    and returning an iterator of the node's children.
    """
    if queue is None:
        queue = []
    queue.append(tree)
    while queue:
        node = queue.pop(0)
        yield node
        if depth != 0:
            try:
                queue += children(node)
                depth -= 1
            except (StopIteration, TypeError, IndexError):
                pass

def iterative_deepening(T, children, callback):
    def visit(node,i):
        if i == 0:
            callback(node)
        else:
            for c in children(node):
                visit(c,i-1)
    i = 0
    while 1:
        visit(T,i)
        i += 1


##def interleave(*iters):
##    """ take several iterators and weave them together, much like roundrobin does except with different ordering. """

# def iter_partition(it, weights, shuffle=None):
#     """ partition an iterator according to weights, return an len(weights) iterators
#
#     ** Do this in an ONLINE FASHION so that we don't need to know the length of the iterator **
#
#     """
#     if shuffle:
#         it = list(it)
#         random.shuffle(it)
#     it = iter(it)


def grouper(n, iterable, fillvalue=None):
    "grouper(3, 'ABCDEFG', 'x') --> ABC DEF Gxx"
    args = [iter(iterable)] * n
    return izip_longest(fillvalue=fillvalue, *args)

def compress(data, selectors):
    "compress('ABCDEF', [1,0,1,0,1,1]) --> A C E F"
    return (d for d, s in izip(data, selectors) if s)

def cross_lower_triangle(it):
    """
    generate pairs of examples which are symmetric and not reflexive

    cross_lower_triangle('abcd')
        | a  b  c  d
     ---|-----------
      a | .  .  .  .
      b | 1  .  .  .
      c | 2  3  .  .
      d | 4  5  6  .
    """
    buf = []
    for x in it:
        for y in buf:
            yield (x,y)
        buf.append(x)

def cross_triangle(it):
    'generate pairs of examples which are symmetric and reflexive'
    buf = []
    for x in it:
        buf.append(x)
        for y in buf:
            yield (x,y)


import heapq
def imerge(*iterables):
    """
    Merge multiple sorted inputs into a single sorted output.

    Equivalent to:  sorted(itertools.chain(*iterables))

    >>> list(imerge([1,3,5,7], [0,2,4,8], [5,10,15,20], [], [25]))
    [0, 1, 2, 3, 4, 5, 5, 7, 8, 10, 15, 20, 25]
    """
    heappop, siftup, _StopIteration = heapq.heappop, heapq._siftup, StopIteration

    h = []
    h_append = h.append
    for it in map(iter, iterables):
        try:
            next = it.next
            h_append([next(), next])
        except _StopIteration:
            pass
    heapq.heapify(h)

    while 1:
        try:
            while 1:
                v, next = s = h[0]      # raises IndexError when h is empty
                yield v
                s[0] = next()           # raises StopIteration when exhausted
                siftup(h, 0)            # restore heap condition
        except _StopIteration:
            heappop(h)                  # remove empty iterator
        except IndexError:
            return


def floor(stream, baseline=None):
    """Generate the stream of minimum values from the input stream.

    The baseline, if supplied, is an upper limit for the floor.
    >>> ff = floor((1, 2, -2, 3))
    >>> assert list(ff) == [1, 1, -2, -2]
    >>> ff = floor((1, 2, -2, 3), 0)
    >>> assert list(ff) == [0, 0, -2, -2]
    """
    stream = iter(stream)
    m = baseline
    if m is None:
        try:
            m = stream.next()
            yield m
        except StopIteration:
            pass
    for s in stream:
        m = min(m, s)
        yield m

def ceil(stream):
    """Generate the stream of maximum values from the input stream.

    >>> top = ceil([0, -1, 2, -2, 3])
    >>> assert list(top) == [0, 0, 2, 2, 3]
    """
    stream = iter(stream)
    try:
        M = stream.next()
        yield M
    except StopIteration:
        pass
    for s in stream:
        M = max(M, s)
        yield M

def iter_length(it):
    """
    Calculate the length or an iterator
    >>> iter_length(xrange(100))
    100
    >>> iter_length(None for _ in xrange(100))
    100
    """
    return sum(1 for _ in it)

def diff(s, t):
    """Generate the differences between two streams

    If the streams are of unequal length, the shorter is truncated.
    >>> dd = diff([2, 4, 6, 8], [1, 2, 3])
    >>> assert list(dd) == [1, 2, 3]
    """
    return imap(sub, s, t)

def last(stream, default=None):
    """Return the last item in the stream or the default if the stream is empty.

    >>> last('abc')
    'c'
    >>> last([], default=-1)
    -1
    """
    s = default
    for s in stream:
        pass
    return s

def accumulate(stream):
    """Generate partial sums from the stream.

    >>> accu = accumulate([1, 2, 3, 4])
    >>> assert list(accu) == [1, 3, 6, 10]
    """
    total = 0
    for s in stream:
        total += s
        yield total


def rolling_average_reccurence(stream):
    """General the rolling average of a stream using
    the recurrence:

        a[i] = (x[i] + i*a) / (i+1)

    >>> list(rolling_average_reccurence(range(5)))
    [0, 0.5, 1.0, 1.5, 2.0]
    """
    stream = enumerate(stream)
    a = stream.next()[1]
    yield a
    for i, x in stream:
        a = (x + i*a) * 1.0 / (i+1)
        yield a


def rolling_average(stream):
    """General the rolling average of a stream.

    >>> list(rolling_average(range(5)))
    [0.0, 0.5, 1.0, 1.5, 2.0]
    """
    acc = 0
    N = 0
    for x in stream:
        acc += x
        N += 1
        yield acc * 1.0 / N


# TODO: can we be lazier? require fewer passes thru X?
def k_fold_cross_validation(X, K, randomize=False):
    """
    Generates K (training, validation) pairs from the items in X.

    Each pair is a partition of X, where validation is an iterable
    of length len(X)/K. So each training iterable is of length (K-1)*len(X)/K.

    If randomise is true, a copy of X is shuffled before partitioning,
    otherwise its order is preserved in training and validation.


    >>> for train, test in k_fold_cross_validation(range(3), 3):
    ...     print 'test:', test, ' train:', list(train)
    test: [0]  train: [1, 2]
    test: [1]  train: [0, 2]
    test: [2]  train: [0, 1]

    """
    if randomize:
        X = list(X)
        shuffle(X)

    # folds = [X[i::K] for i in xrange(K)]

    # run thru X once and splitting into folds
    folds = [[] for k in xrange(K)]
    for i, item in enumerate(X):
        folds[i % K].append(item)

    for k in xrange(K):
        training = chain(*(fold for j, fold in enumerate(folds) if j != k))
        yield training, folds[k]

#deprecated use itertools.product instead
#def xCross(sets, *more):
#    """
#    Take the cartesian product of two or more iterables.
#    The input can be either one argument, a collection of iterables xCross([it1,it2,...]),
#    or several arguments, each one an iterable xCross(it1, itb, ...).
#
#    >>> [(x,y,z) for x,y,z in xCross([1,2], 'AB', [5])]
#    [(1, 'A', 5), (1, 'B', 5), (2, 'A', 5), (2, 'B', 5)]
#
#    """
#    if more:
#        sets = chain((sets,), more)
#    sets = list(sets)
#    wheels = map(iter, sets) # wheels like in an odometer
#    digits = [it.next() for it in wheels]
#    while True:
#        yield digits[:]
#        for i in range(len(digits)-1, -1, -1):
#            try:
#                digits[i] = wheels[i].next()
#                break
#            except StopIteration:
#                wheels[i] = iter(sets[i])
#                digits[i] = wheels[i].next()
#        else:
#            break


def cross_product(A,B):
    """ take the cartesian product of two iterables A and B. """
    for a in A:
        for b in B:
            yield (a,b)


def unique(iterable, key=None):
    """ List unique elements, preserving order. Remember all elements ever seen. """
    # unique_everseen('AAAABBBCCDAABBB') --> A B C D
    # unique_everseen('ABBCcAD', str.lower) --> A B C D
    seen = set()
    seen_add = seen.add
    if key is None:
        for element in iterable:
            if element not in seen:
                seen_add(element)
                yield element
    else:
        for element in iterable:
            k = key(element)
            if k not in seen:
                seen_add(k)
                yield element


# Recipe credited to George Sakkis
def roundrobin(*iterables):
    """ roundrobin('ABC', 'D', 'EF') --> A D E B F C

    >>> list(roundrobin('ABC', 'D', 'EF'))
    ['A', 'D', 'E', 'B', 'F', 'C']
    """
    pending = len(iterables)
    nexts = cycle(iter(it).next for it in iterables)
    while pending:
        try:
            for next in nexts:
                yield next()
        except StopIteration:
            pending -= 1
            nexts = cycle(islice(nexts, pending))


def window(iterable, k):
    """
    s -> (s[0],...,s[k]) (s[1],...,s[k+1]) ... (s[i],...,s[i+k]) ... (s[n-1-k],...,s[n-1])

    >>> [x+y+z for x,y,z in window('abcdef', 3)]
    ['abc', 'bcd', 'cde', 'def']

    """
    iterators = tee(iterable, k)
    for i, it in enumerate(iterators):
        # advance iterator, 'it',  by i steps
        for _ in xrange(i):
            try:
                it.next()    # advance by one
            except StopIteration:
                pass
    return izip(*iterators)

sliding_window = window


def iterview(x, every=1, msg='', length=None):
    """
    iterator which prints its progress to *stderr*.
    """
    WIDTH = 70

    def plainformat(n, lenx):
        return '%5.1f%% (%*d/%d)' % ((float(n)/lenx)*100, len(str(lenx)), n, lenx)

    def bars(size, n, lenx):
        val = int((float(n)*size)/lenx + 0.5)
        if size - val:
            spacing = ">" + (" "*(size-val))[1:]
        else:
            spacing = ""
        return "[%s%s]" % ("="*val, spacing)

    def eta(elapsed, n, lenx):
        if n == 0:
            return '--:--:--'
        if n == lenx:
            secs = int(elapsed)
        else:
            secs = int((elapsed/n) * (lenx-n))
        mins, secs = divmod(secs, 60)
        hrs, mins = divmod(mins, 60)
        return '%02d:%02d:%02d' % (hrs, mins, secs)

    def fmt(starttime, n, lenx):
        out = ''
        if msg:
            out += msg + ' '
        out += plainformat(n, lenx) + ' '
        if n == lenx:
            end = '     '
        else:
            end = ' ETA '
        end += eta(time() - starttime, n, lenx)
        out += bars(WIDTH - len(out) - len(end), n, lenx)
        out += end
        return out

    starttime = time()
    lenx = length or len(x)
    n = lenx
    if lenx == 0:
        raise StopIteration

    #history = []
    for n, y in enumerate(x):
        if every is None or n % every == 0:
            sys.stderr.write('\r' + fmt(starttime, n, lenx))

        #b4 = time()
        yield y

        #history.append(time() - b4)
        #pylab.ion()
        #pylab.hist(history, bins=50)
        #pylab.draw()

    sys.stderr.write('\r' + fmt(starttime, n+1, lenx) + '\n')

#import pylab

#_______________________________________________________________________________
#

def consume(iterator, n):
    "Advance the iterator n-steps ahead. If n is none, consume entirely."
    # The technique uses objects that consume iterators at C speed.
    if n is None:
        # feed the entire iterator into a zero-length deque
        deque(iterator, maxlen=0)
    else:
        # advance to the emtpy slice starting at position n
        next(islice(iterator, n, n), None)

def take(n, seq):
    """ Return the first n items in a sequence. """
    return list(islice(seq, None, n))

def nth(iterable, n):
    """ Returns a list containing the nth item. """
    return list(islice(iterable, n, n+1))

def no(seq, pred=None):
    """
    the opposite of all, returns True if pred(x) is false for every element
    in the iterable.
    """
    for elem in ifilter(pred, seq):
        return False
    return True

def quantify(seq, pred=None):
    """ Count how many times the predicate is true in the sequence """
    return sum(imap(pred, seq))

def padnone(seq):
    """ Returns the sequence elements and then returns None indefinitely """
    return chain(seq, repeat(None))

def ncycles(seq, n):
    """ Returns the sequence elements n times """
    return chain(*repeat(seq, n))

def dotproduct(vec1, vec2):
    """ Return the dot product between two vectors. """
    return sum(imap(mul, vec1, vec2))

def flatten(listOfLists):
    """ (non-recursive) Flatten a list of lists. """
    return list(chain(*listOfLists))

def batch(iterable, batchsize=2):
    """Yield a list of (up to) batchsize items at a time.  The last
    element will be shorter if there are items left over.
    batch(s, 2) -> [s0,s1], [s2,s3], [s4, s5], ...

    >>> list(batch(range(5), 2))
    [[0, 1], [2, 3], [4]]
    """
    current = []
    for item in iterable:
        current.append(item)
        if len(current) == batchsize:
            yield current
            current = []
    if current:
        yield current

def full_batches(iterable, batchsize=2):
    """ Differs from batch in that it returns only batchs of size equal to batchsize, no less.

    >>> list(full_batches(range(5), 2))
    [(0, 1), (2, 3)]
    """
    return izip(*repeat(iter(iterable), batchsize))

def batch_extra_lazy(iterable, batchsize):
    """ batch(s, 2) -> [s0,s1], [s2,s3], [s4, s5], ...

    Yield a list of (up to) batchsize items at a time.

    >>> map(tuple, batch_extra_lazy(range(5), 2))
    [(0, 1), (2, 3), (4,)]
    """
    sourceiter = iter(iterable)
    while True:
        batchiter = islice(sourceiter, batchsize)
        yield chain([batchiter.next()], batchiter)


def iunzip(iterable, n=None):
    """Takes an iterator that yields n-tuples and returns n iterators
    which index those tuples.  This function is the reverse of izip().
    n is the length of the n-tuple and will be autodetected if not
    specified.  If the iterable contains tuples of differing sizes,
    the behavior is undefined."""
    # a braindead implementation for now (since it relies on tee() which is
    # braindead in this module (but not in Python 2.4+))
    iterable = iter(iterable) # ensure we're dealing with an iterable
    if n is None: # check the first element for length
        first = iterable.next()
        n = len(first)
        # now put it back in to iterable is unchanged
        iterable = chain([first], iterable)

    iter_tees = tee(iterable, n)
    selector = lambda index: lambda item: getitem(item, index)
    return [imap(selector(i), it) for i,it in izip(count(), iter_tees)]


if __name__ == "__main__":

    def test():
        a0,b0,c0 = range(1, 10), range(21, 30), range(81, 90)
        test = zip(a0,b0,c0)
        a, b, c = iunzip(test)
        a, b, c = map(list, (a,b,c))
        assert a == a0 and b == b0 and c == c0
        recombined = zip(a, b, c)
        assert recombined == test

        def example_iterview():
            for _ in iterview(xrange(400), every=20):
                sleep(0.01)
        #example_iterview()

        X = [[0 for i in xrange(4)] for j in xrange(4)]
        for (k,(i,j)) in enumerate(cross_lower_triangle(range(4))):
            X[i][j] = k+1
        target = [[0, 0, 0, 0],
                  [1, 0, 0, 0],
                  [2, 3, 0, 0],
                  [4, 5, 6, 0]]
        assert X == target

        import numpy as np
        d = (np.random.rand(20, 3) - 0.5) * 100
        A = np.average(d, axis=0)
        a = last(rolling_average(d))
        assert np.linalg.norm(A - a) < 1e-10  # roughly zero difference

        import doctest; doctest.testmod()

        for _ in iterview(range(100), 1):
            sleep(.1)

    test()



stopwords = """
a
aboard
about
above
across
after
against
all
along
alongside
although
amid
amidst
among
amongst
an
and
another
anti
any
anybody
anyone
anything
around
as
astride
at
aught
bar
barring
because
before
behind
below
beneath
beside
besides
between
beyond
both
but
by
circa
concerning
considering
despite
down
during
each
either
enough
everybody
everyone
except
excepting
excluding
few
fewer
following
for
from
he
her
hers
herself
him
himself
his
hisself
i
idem
if
ilk
in
including
inside
into
it
its
itself
like
many
me
mine
minus
more
most
myself
naught
near
neither
nobody
none
nor
nothing
notwithstanding
of
off
on
oneself
onto
opposite
or
other
otherwise
our
ourself
ourselves
outside
over
own
past
pending
per
plus
regarding
round
save
self
several
she
since
so
some
somebody
someone
something
somewhat
such
suchlike
sundry
than
that
the
thee
theirs
them
themselves
there
they
thine
this
thou
though
through
throughout
thyself
till
to
tother
toward
towards
twain
under
underneath
unless
unlike
until
up
upon
us
various
versus
via
vis-a-vis
we
what
whatall
whatever
whatsoever
when
whereas
wherewith
wherewithal
which
whichever
whichsoever
while
who
whoever
whom
whomever
whomso
whomsoever
whose
whosoever
with
within
without
worth
ye
yet
yon
yonder
you
you-all
yours
yourself
yourselves
"""



import re
from arsenal.misc import force
from itertools import imap

class ParseError(Exception):
    """ Custom exception class used by this module. """
    pass

class Span(object):
    __slots__ = ('label','begins','ends')
    def __init__(self, label, begins, ends):
        self.label = label
        self.begins = begins
        self.ends = ends
    def __repr__(self):
        return 'Span(label=%r, begins=%r, ends=%r)' % (self.label, self.begins, self.ends)
    def __eq__(self, other):
        if isinstance(other, Span):
            return (self.label == other.label and self.begins == other.begins and self.ends == other.ends)
        else:
            return len(other) == 3 and (self.label == other[0] and self.begins == other[1] and self.ends == other[2])
    def __iter__(self):
        return iter((self.label, self.begins, self.ends))


WhitespaceLexer = re.compile('\S+')

Lexer = re.compile('|'.join(["http://\S+",                               # keep urls together.. might include invalid URL characters
                             "\S+@\S+",
                             "[0-9][0-9]?\s*\([0-9]?\)",                 # e.g. "7(4)" common in volume
                             "\(\s*[0-9][0-9]?\s*\)",                    # e.g. "(4)" common in volume
                             "[0-9]+\s*-\s*[0-9]+",                      # possible page number
                             "[0-9]+(?:st|nd|rd|th)",
                             "[A-Z]\.",
                             "Ph\.?[Dd]\.",
                             #"[Vv]ol(?:\.|ume)\s*[0-9]+",               # make "Vol. 3" one token.
                             "(?:Vol|Proc|Dept|Univ|No|Inc|Dr)\s*\.",
                             "pp\.",
                             "\(\s*[0-9][0-9][0-9][0-9][a-z]?\s*\)",     # e.g. "(1994)" year in parens
                             "[Ee]d(?:s?\.|itors?)",
                             #"\w+-\w+",
                             "\+[A-Z]+\+",
                             "[a-zA-Z]+(?:'s)?",
                             #"[0-9]+(?:\.[0-9]+)?",                      # 231 or 2.0, but not 2.
                             "[0-9]+",
                             "[()\"'\-\.,]",
                             "\S+"]))

TaggedText = re.compile("<([a-z0-9_]+)>([\w\W]+?)</([a-z0-9_]+)>|([^<>\s]+)", re.IGNORECASE)

def fromSGML(f, linegrouper="\n", bioencoding=False):
    for line in re.split(linegrouper, open(f).read()):
        if bioencoding:
            seq = sgml2bio(line)
        else:
            seq = sgml2seq(line)
        if seq:
            yield seq

@force
def sgml2segmentation(x, lexer=WhitespaceLexer):
    """
    >>> sgml2segmentation('<title>Cat in the Hat</title><author>Dr. Seuss</author>')
    [('title', ['Cat', 'in', 'the', 'Hat']), ('author', ['Dr.', 'Seuss'])]
    """
    x = x.strip().replace("\n", " +L+ ")
    for (tag, tagged, close, outside) in TaggedText.findall(x):
        if tag != close:
            raise ParseError("opening (%s) and closing (%s) tags do not match in sequence\n    %r\n" % (tag, close, x))
        if tagged:
            yield (tag, lexer.findall(tagged))
        else:
            for w in lexer.findall(outside):
                yield ("O", [w])

@force
def sgml2bio(x):
    """
    >>> sgml2bio('<title>Cat in the Hat</title><author>Dr. Seuss</author>')
    [('B-title', 'Cat'), ('I-title', 'in'), ('I-title', 'the'), ('I-title', 'Hat'), ('B-author', 'Dr.'), ('I-author', 'Seuss')]
    """
    for (tag, tokens) in sgml2segmentation(x):
        tokens = iter(tokens)
        yield ('B-' + tag, tokens.next())
        for w in tokens:
            yield ('I-' + tag, w)

@force
def sgml2seq(x):
    """
    >>> sgml2seq('<title>Cat in the Hat</title><author>Dr. Seuss</author>')
    [('title', 'Cat'), ('title', 'in'), ('title', 'the'), ('title', 'Hat'), ('author', 'Dr.'), ('author', 'Seuss')]
    """
    for (tag, tokens) in sgml2segmentation(x):
        for w in tokens:
            yield (tag, w)

def bracket2bio(x):
    """
    generate BIO-token pairs from bracket-style annotation.
    Note: splits text of spaces, so wordsplitting should already be done.

    >>> x = bracket2bio("[TITLE Cat in the Hat][AUTHOR Dr. Seuss]")
    >>> list(x)                                  #doctest:+NORMALIZE_WHITESPACE
    [('B-TITLE', 'Cat'), ('I-TITLE', 'in'), ('I-TITLE', 'the'),
     ('I-TITLE', 'Hat'), ('B-AUTHOR', 'Dr.'), ('I-AUTHOR', 'Seuss')]
    """
    if '\n' in x:
        raise ParseError('No newlines allowed in brack2bio annotation.')
    for label, tagged, word in re.findall('(?:(?:\[([A-Z0-9]+)\s+(.+?)\s*\]\s*)|(.+?)(?:\s+|$))', x):
        if word:
            yield ('O', word)
        else:
            words = iter(tagged.split())
            yield ('B-%s' % label, words.next())
            for w in words:
                if '[' in w or ']' in w:
                    raise ParseError('brackets can not appear within a word.')
                yield ('I-%s' % label, w)

# TIMV: we want something like a LineGroupIterator
def line_groups(text, pattern):
    """
    Very simple function for breaking up text into groups based on a
    single pattern.

    >>> list(line_groups("a BB c d BB", "BB"))
    ['a', 'c d']
    """
    for group in re.split(pattern, text):    # TODO: make this "lazier"
        group = group.strip()
        if group:
            yield group


def extract_contiguous(s, labeler=None):
    """
    >>> list(extract_contiguous(""))
    []

    >>> list(extract_contiguous("AABBC"))
    [Span(label='A', begins=0, ends=2), Span(label='B', begins=2, ends=4), Span(label='C', begins=4, ends=5)]

    >>> list(extract_contiguous("AABBB"))
    [Span(label='A', begins=0, ends=2), Span(label='B', begins=2, ends=5)]
    """
    if labeler is not None:
        s = imap(labeler, s)
    prev = None
    b = e = 0
    for e, token in enumerate(s):
        if token != prev:
            if prev is not None:
                yield Span(prev, b, e)
            b = e
        prev = token
    # emit lingering bits
    if prev is not None:
        yield Span(prev, b, e + 1)


@force
def bio2span(seq, tagger=None, include_O=True):
    if tagger is not None:
        seq = imap(tagger, seq)
    phrase = None
    intag = None
    for i, lbl in enumerate(seq):
        if lbl is None:
            lbl = 'O'
        label = lbl[2:]
        if lbl.startswith('B-'):
            if intag and phrase:
                yield phrase
            phrase = Span(label, i, i + 1)
        elif lbl.startswith('I-'):
            if intag == label:             # and youre still in the same span
                phrase.ends = i + 1
            else:                          # you're in a new span (hueristic correction)
                if phrase:
                    yield phrase
                phrase = Span(label, i, i + 1)
        else:
            if intag:                      # was in tag, now outiside ("O")
                if phrase:
                    yield phrase
                phrase = None
            if include_O:
                yield Span(lbl, i, i + 1)
        if lbl == 'O':
            intag = None
        else:
            intag = label
    if intag and phrase:                   # close any lingering spans
        yield phrase


if __name__ == '__main__':
    from arsenal.misc import piped
    def main():
        for line in piped() or []:
            for (label, w) in sgml2bio(line):
                print '%s\t%s' % (label, w)
            print
    main()



# -*- coding: utf-8 -*-

"""
The wordsplitting is based on the wordsplitting perl script provided by the
Cognitive Computation Group at the University of Illinois at Champaign-Urbana.
"""

import re

###
# IDEAS/TODO:
#    Nets' => Net 's
#

# Separate numbers from words.
def numbers_words(m):
    a,b = m.groups()
    if b.lower() in ['st','nd','rd','th']:
        return '%s%s' % (a,b)
    else:
        return '%s %s' % (a,b)


def wordsplit_sentence(sentence):
    # Convert some special characters to ASCII to ensure they aren't lost later.
    sentence = re.sub('(\xe2\x82\xac|%u20AC)', 'Euros', sentence)
    sentence = re.sub('(\xc3\x82\xc2\xa3|\xc2\xa3|\xa31|\xc2\xa3|GBP)', ' GBP ', sentence)

    # take a look at python-extras/unicode_annoyances.py

    # remove these...
    #sentence = re.sub('(\xc3\xab|\xc3\xa3)','', sentence)
    sentence = re.sub('[^\x20-\x7E]', '', sentence)   ####### <<<< KILL non-nice chars.

    sentence = re.sub('@', 'AAATTT', sentence)

    # ASCII HACKS!
    #for hexcode, ascii in LATIN2ASCII.iteritems():
    #    try:
    #        sentence = sentence.replace(unichr(hexcode), ascii)
    #    except UnicodeDecodeError:
    #        pass
    #sentence = str(sentence)

    # Replace repeated punctuation marks with something equivalent.  These
    # replacements also make simplifying assumptions that will become useful later
    # in this function.
    sentence = re.sub('\-\-*', '-', sentence)

    ###########################################
    # fix escape codes
    sentence = re.sub('%u2013', '-', sentence)
    ###########################################

    before = ''
    while before != sentence:
        before = sentence
        # Look for closing quotes
        sentence = re.sub('\'\'([^\'\w]|$)', r'" \1', sentence)
        # opening quotes
        sentence = re.sub('(^|[^\'\w\.\,\:\;\!\?])\'\'', r'\1 "', sentence)

    # Remove leading and trailing whitespace.
    sentence = sentence.strip()

    # Separate punctuation marks from each other.
    sentence = re.sub('([^\w\s\`])([^\w\s\`])', r'\1 \2', sentence)

    # Separate single quotes that don't look like apostrophes.
    sentence = re.sub('(^|\W)(\')(\w)', r'\1\2 \3', sentence)
    sentence = re.sub('(\w)(\')(\W|$)', r'\1 \2\3', sentence)

    # we want 's not ' s
    sentence = re.sub("(\w)\s*'\s*s(\W|$)", r"\1 's \2", sentence)

    # separate contractions
    sentence = re.sub('(\S)([^\w\s\`\.\,\-$])', r'\1 \2', sentence)

    sentence = re.sub('([^\w\s\`\'\.\,\-])(\S)', r'\1 \2', sentence)

    # Separate opening single quotes from everything else, except keep repeated
    # opening single quotes in pairs.
    sentence = re.sub('([^\`])(\`)', r'\1 \2', sentence)
    sentence = re.sub('(\`)([^\`])', r'\1 \2', sentence)

    before = ''
    while before != sentence:
        before = sentence
        sentence = re.sub('(^|\s)\`\`\`', r'\1\`\` \`', sentence)

    # Separate stray dashes when they don't seem to be connecting words usefully.
    sentence = re.sub('(\S)(\-)(\s|$)', r'\1 \2\3', sentence)
    sentence = re.sub('(^|\s)(\-)(\S)', r'\1\2 \3', sentence)

    # ???: always separate the dash?
    sentence = re.sub('(\-)', r' - ', sentence)

    # Separate commas from words, but not from within numbers.
    sentence = re.sub('(\S),(\s|$)', r'\1 ,\2', sentence)
    sentence = re.sub('(^|\s),(\S)', r'\1, \2', sentence)
    sentence = re.sub('(\D),(\S)', r'\1 , \2', sentence)
    sentence = re.sub('(\S),(\D)', r'\1 , \2', sentence)

    sentence = re.sub('(\d)\s*([^\W\d]+)', numbers_words, sentence)
    sentence = re.sub('([^\W\d])(\d)', r'\1 \2', sentence)

    # SMOOSH times together
    sentence = re.sub('(\d\d?)\s*:\s*(\d\d)(\W|$)', r'\1:\2\3', sentence)

    # keep things that look like abbrev, initial, honorific together
    sentence = re.sub('(^|\s)([A-Z][a-z]*)\s*\.', r'\1\2.', sentence)

    # weird thing that happens often with dates...
    sentence = re.sub('(\d),(\d{4,})(\W)', r'\1 , \2\3', sentence)

    # WARNING: I did more that word split here!
    # its that bizzare european thing...
    sentence = re.sub('(\d),(\d{2})(\W)', r'\1.\2 \3', sentence)

    # Separate words from closing punctuation (i.e., the last char in input).
    sentence = re.sub('(\w)(\.)(\W*)$', r'\1 \2 \3', sentence)

    # 1990 s -> 1990s; 90 s -> 90s
    sentence = re.sub('(\d\d\d\d|\d\d)\s*s(\s+|$)', r'\1s ', sentence)

    # ' 90 -> '90
    sentence = re.sub("'\s*(\d\d)($|\W)", r"'\1\2", sentence)

    #######
    # Tags will explode, so, as a last step,
    #  * tighten up opening tags
    sentence = re.sub('<\s*([a-zA-z_\-0-9]+?)\s*>', r'<\1>', sentence)
    #  * tighten up closing tags.
    sentence = re.sub('<\s*/\s*([a-zA-z_\-0-9]+?)\s*>', r'</\1>', sentence)
    #  * tighten-up to bracketed tags too.
    sentence = re.sub('\[\s*([A-z]+?)\s', r'[\1 ', sentence)
    #  * tighten up the +L+ line break marker
    sentence = re.sub('\+\s*L\s*\+', r'+L+', sentence)
    #######

    sentence = re.sub('AAATTT', r'@', sentence)


    ## fixed escapes
    sentence = re.sub('x\d\d \s \\ x\d\d', r'\1\2', sentence, re.VERBOSE|re.IGNORECASE)

    # tighten up possible space explosion
    sentence = re.sub('[ ]+', ' ', sentence)

    return sentence


if __name__ == '__main__':
    tests = [
        'I lost 500US$ (EU800) in March 2008.',
        'I lost $500 (EU800) in March 2008.',
        'I lost 500US$ (EU800) in Mar. 18th 2008.',
        'I lost US$500.34 (EU800) in Mar. 18th 2008.',
        'I lost <NUM> 500US$ </NUM> (EU800) in Mar. 18th 2008.',
        'I lost the [NUM 50-meter] run (EU800) in Mar. 18th 2008.',
        'I lost the 55,55 in Mar. 18,2008.',
        'I lost the 55,55 in Mar . 18,2008.8.',
        'I lost the 5:30 in Mar. 18,2008.8.',
        'I lost the 5 : 30 in Mar . 18,2008.8.',
        'Dr. barry\'s . Dr . barry \'s . barry \' s .',
        'It was just [NUM .03 seconds ] back to ',
        '1990 s 1990s \' 90 \'90',
        'In 2002, 708,083 sexually transmitted infections were reported',
    ]

    for t in tests:
        print t
        print wordsplit_sentence(t)
        print




# Extracted from pandas, which intern extracted it from
# https://gist.github.com/huyng/816622 this is the rcParams set when setting
# display.with_mpl_style to True.
mpl_stylesheet = {
    'axes.axisbelow': True,
     'axes.color_cycle': ['#348ABD',
                          '#7A68A6',
                          '#A60628',
                          '#467821',
                          '#CF4457',
                          '#188487',
                          '#E24A33'],
     'axes.edgecolor': '#bcbcbc',
     'axes.facecolor': '#eeeeee',
     'axes.grid': True,
     'axes.labelcolor': '#555555',
     'axes.labelsize': 'large',
     'axes.linewidth': 1.0,
     'axes.titlesize': 'x-large',
     'figure.edgecolor': 'white',
     'figure.facecolor': 'white',
     'figure.figsize': (6.0, 4.0),
     'figure.subplot.hspace': 0.5,
     'font.family': 'monospace',
     'font.monospace': ['Andale Mono',
                        'Nimbus Mono L',
                        'Courier New',
                        'Courier',
                        'Fixed',
                        'Terminal',
                        'monospace'],
     'font.size': 8,
     'interactive': True,
     'keymap.all_axes': ['a'],
     'keymap.back': ['left', 'c', 'backspace'],
     'keymap.forward': ['right', 'v'],
     'keymap.fullscreen': ['f'],
     'keymap.grid': ['g'],
     'keymap.home': ['h', 'r', 'home'],
     'keymap.pan': ['p'],
     'keymap.save': ['s'],
     'keymap.xscale': ['L', 'k'],
     'keymap.yscale': ['l'],
     'keymap.zoom': ['o'],
     'legend.fancybox': True,
     'lines.antialiased': True,
     'lines.linewidth': 1.0,
     'patch.antialiased': True,
     'patch.edgecolor': '#EEEEEE',
     'patch.facecolor': '#348ABD',
     'patch.linewidth': 0.5,
     'toolbar': 'toolbar2',
     'xtick.color': '#555555',
     'xtick.direction': 'in',
     'xtick.major.pad': 6.0,
     'xtick.major.size': 0.0,
     'xtick.minor.pad': 6.0,
     'xtick.minor.size': 0.0,
     'ytick.color': '#555555',
     'ytick.direction': 'in',
     'ytick.major.pad': 6.0,
     'ytick.major.size': 0.0,
     'ytick.minor.pad': 6.0,
     'ytick.minor.size': 0.0
}

if __name__ == '__main__':
    import numpy as np
    import matplotlib as mpl
    import matplotlib.pylab as pl

    mpl.rc_params().update(mpl_stylesheet)

    xs, ys = np.random.uniform(0,1,size=(2,30))
    pl.scatter(xs, ys)

    pl.show()



#!/usr/bin/python
# By Steve Hanov, 2011. Released to the public domain.
import sys
import time

class DawgNode(object):
    """
    This class represents a node in the directed acyclic word graph (DAWG). It
    has a list of edges to other nodes. It has functions for testing whether it
    is equivalent to another node. Nodes are equivalent if they have identical
    edges, and each identical edge leads to identical states. The __hash__ and
    __eq__ functions allow it to be used as a key in a python dictionary.
    """

    NextId = 0

    def __init__(self):
        self.id = DawgNode.NextId
        DawgNode.NextId += 1
        self.final = False
        self.edges = {}

    def __str__(self):
        arr = []
        if self.final:
            arr.append("1")
        else:
            arr.append("0")
        for (label, node) in self.edges.iteritems():
            arr.append(label)
            arr.append(str(node.id))
        return '_'.join(arr)

    def __hash__(self):
        return hash(str(self))

    def __eq__(self, other):
        return str(self) == str(other)


class Dawg(object):

    def __init__(self):
        self.previousWord = ''
        self.root = DawgNode()

        # Here is a list of nodes that have not been checked for duplication.
        self.uncheckedNodes = []

        # Here is a list of unique nodes that have been checked for
        # duplication.
        self.minimizedNodes = {}

    def insert(self, word):
        if word < self.previousWord:
            raise Exception("Words must be inserted in alphabetical order.")

        # find common prefix between word and previous word
        commonPrefix = 0
        for i in xrange(min(len(word), len(self.previousWord))):
            if word[i] != self.previousWord[i]:
                break
            commonPrefix += 1

        # Check the uncheckedNodes for redundant nodes, proceeding from last
        # one down to the common prefix size. Then truncate the list at that
        # point.
        self._minimize(commonPrefix)

        # add the suffix, starting from the correct node mid-way through the
        # graph
        if len(self.uncheckedNodes) == 0:
            node = self.root
        else:
            node = self.uncheckedNodes[-1][2]

        for letter in word[commonPrefix:]:
            nextNode = DawgNode()
            node.edges[letter] = nextNode
            self.uncheckedNodes.append( (node, letter, nextNode) )
            node = nextNode

        node.final = True
        self.previousWord = word

    def finish(self):
        # minimize all uncheckedNodes
        self._minimize(0)

    def _minimize(self, downTo):
        # proceed from the leaf up to a certain point
        for i in range(len(self.uncheckedNodes) - 1, downTo - 1, -1):
            (parent, letter, child) = self.uncheckedNodes[i]
            if child in self.minimizedNodes:
                # replace the child with the previously encountered one
                parent.edges[letter] = self.minimizedNodes[child]
            else:
                # add the state to the minimized nodes.
                self.minimizedNodes[child] = child
            self.uncheckedNodes.pop()

    def lookup(self, word):
        node = self.root
        for letter in word:
            if letter not in node.edges:
                return False
            node = node.edges[letter]
        return node.final

def dawg(words):
    d = Dawg()
    words.sort()
    for w in words:
        d.insert(w)
    d.finish()
    return d

def main(dictionary, *queries):
    dictionary = dictionary or '/usr/share/dict/words'

    words = open(dictionary, 'rt').read().split()
    start = time.time()
    d = dawg(words)

    print "Dawg creation took %g s" % (time.time() - start)
    print "Read %d words into %d nodes and %d edges" % (len(words),
                                                        len(d.minimizedNodes),
                                                        sum(len(node.edges) for node in d.minimizedNodes))

    for word in queries:
        if not d.lookup(word):
            print "%s not in dictionary." % word
        else:
            print "%s is in the dictionary." % word

if __name__ == '__main__':
    main(*sys.argv[1:])



#!/usr/bin/env python

#****************************************************************************
# unitgroup.py, provides a group of units and does conversions
#
# ConvertAll, a units conversion program
# Copyright (C) 2006, Douglas W. Bell
#
# This is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License, Version 2.  This program is
# distributed in the hope that it will be useful, but WITTHOUT ANY WARRANTY.
#*****************************************************************************

import re
from math import *
from unitatom import UnitAtom


class UnitGroup:
    """Stores, updates and converts a group of units"""
    maxDecPlcs = 12
    operRegEx = re.compile(r'([\*/])')
    def __init__(self, unitData, option):
        self.unitData = unitData
        self.option = option
        self.unitList = []
        self.currentNum = 0
        self.factor = 1.0
        self.reducedList = []
        self.linear = True

    def update(self, text, cursorPos=None):
        """Decode user entered text into units"""
        self.unitList = self.parseGroup(text)
        if cursorPos != None:
            self.updateCurrentUnit(text, cursorPos)
        else:
            self.currentNum = len(self.unitList) - 1

    def updateCurrentUnit(self, text, cursorPos):
        """Set current unit number"""
        self.currentNum = len(UnitGroup.operRegEx.findall(text[:cursorPos]))

    def currentUnit(self):
        """Return current unit if its a full match, o/w None"""
        if self.unitList and self.unitList[self.currentNum].equiv:
            return self.unitList[self.currentNum]
        return None

    def currentPartialUnit(self):
        """Return unit with at least a partial match, o/w None"""
        if not self.unitList:
            return None
        return self.unitData.findPartialMatch(self.unitList[self.currentNum]\
                                              .name)

    def currentSortPos(self):
        """Return unit near current unit for sorting"""
        if not self.unitList:
            return self.unitData[self.unitData.sortedKeys[0]]
        return self.unitData.findSortPos(self.unitList[self.currentNum]\
                                         .name)

    def replaceCurrent(self, unit):
        """Replace the current unit with unit"""
        if self.unitList:
            exp = self.unitList[self.currentNum].exp
            self.unitList[self.currentNum] = unit.copy()
            self.unitList[self.currentNum].exp = exp
        else:
            self.unitList.append(unit.copy())

    def completePartial(self):
        """Replace a partial unit with a full one"""
        if self.unitList and not self.unitList[self.currentNum].equiv:
            text = self.unitList[self.currentNum].name
            unit = self.unitData.findPartialMatch(text)
            if unit:
                self.replaceCurrent(unit)

    def moveToNext(self, upward):
        """Replace unit with adjacent one based on match or sort position"""
        unit = self.currentSortPos()
        name = unit.name.lower().replace(' ', '')
        num = self.unitData.sortedKeys.index(name) + (upward and -1 or 1)
        if 0 <= num < len(self.unitData.sortedKeys):
            self.replaceCurrent(self.unitData[self.unitData.sortedKeys[num]])

    def addOper(self, mult):
        """Add new operator & blank unit after current, * if mult is true"""
        if self.unitList:
            self.completePartial()
            prevExp = self.unitList[self.currentNum].exp
            self.currentNum += 1
            self.unitList.insert(self.currentNum, UnitAtom(''))
            if (not mult and prevExp > 0) or (mult and prevExp < 0):
                self.unitList[self.currentNum].exp = -1

    def changeExp(self, newExp):
        """Change the current unit's exponent"""
        if self.unitList:
            self.completePartial()
            if self.unitList[self.currentNum].exp > 0:
                self.unitList[self.currentNum].exp = newExp
            else:
                self.unitList[self.currentNum].exp = -newExp

    def clearUnit(self):
        """Remove units"""
        self.unitList = []
        self.currentNum = 0
        self.factor = 1.0
        self.reducedList = []
        self.linear = True

    def parseGroup(self, text):
        """Return list of units from text string"""
        unitList = []
        parts = [part.strip() for part in UnitGroup.operRegEx.split(text)]
        numerator = True
        while parts:
            unit = self.parseUnit(parts.pop(0))
            if not numerator:
                unit.exp = -unit.exp
            if parts and parts.pop(0) == '/':
                numerator = not numerator
            unitList.append(unit)
        return unitList

    def parseUnit(self, text):
        """Return a valid or invalid unit with exponent from a text string"""
        parts = text.split('^', 1)
        exp = 1
        if len(parts) > 1:   # has exponent
            try:
                exp = int(parts[1])
            except ValueError:
                if parts[1].lstrip().startswith('-'):
                    exp = -UnitAtom.partialExp  # tmp invalid exp
                else:
                    exp = UnitAtom.partialExp
        unitText = parts[0].strip().lower().replace(' ', '')
        unit = self.unitData.get(unitText, None)
        if not unit and unitText and unitText[-1] == 's' and not \
           self.unitData.findPartialMatch(unitText):   # check for plural
            unit = self.unitData.get(unitText[:-1], None)
        if not unit:
            unit = UnitAtom(parts[0].strip())   # tmp invalid unit
        unit = unit.copy()
        unit.exp = exp
        return unit

    def unitString(self, unitList=None):
        """Return the full string for this group or a given group"""
        if unitList == None:
            unitList = self.unitList
        fullText = ''
        if unitList:
            fullText = unitList[0].unitText(0)
            numerator = True
            for unit in unitList[1:]:
                if (numerator and unit.exp > 0) \
                   or (not numerator and unit.exp < 0):
                    fullText = '%s * %s' % (fullText, unit.unitText(1))
                else:
                    fullText = '%s / %s' % (fullText, unit.unitText(1))
                    numerator = not numerator
        return fullText

    def groupValid(self):
        """Return True if all unitself.reducedLists are valid"""
        if not self.unitList:
            return False
        for unit in self.unitList:
            if not unit.unitValid():
                return False
        return True

    def reduceGroup(self):
        """Update reduced list of units and factor"""
        self.linear = True
        self.reducedList = []
        self.factor = 1.0
        if not self.groupValid():
            return
        count = 0
        tmpList = self.unitList[:]
        while tmpList:
            count += 1
            if count > 5000:
                raise UnitDataError, 'Circular unit definition'
            unit = tmpList.pop(0)
            if unit.equiv == '!':
                self.reducedList.append(unit.copy())
            elif not unit.equiv:
                raise UnitDataError, 'Invalid conversion for "%s"' % unit.name
            else:
                if unit.fromEqn:
                    self.linear = False
                newList = self.parseGroup(unit.equiv)
                for newUnit in newList:
                    newUnit.exp *= unit.exp
                tmpList.extend(newList)
                self.factor *= unit.factor**unit.exp
        self.reducedList.sort()
        tmpList = self.reducedList[:]
        self.reducedList = []
        for unit in tmpList:
            if self.reducedList and unit == self.reducedList[-1]:
                self.reducedList[-1].exp += unit.exp
            else:
                self.reducedList.append(unit)
        self.reducedList = [unit for unit in self.reducedList if \
                            unit.name != 'unit' and unit.exp != 0]

    def categoryMatch(self, otherGroup):
        """Return True if unit types are equivalent"""
        if not self.checkLinear() or not otherGroup.checkLinear():
            return False
        return self.reducedList == otherGroup.reducedList and \
               [unit.exp for unit in self.reducedList] \
               == [unit.exp for unit in otherGroup.reducedList]

    def checkLinear(self):
        """Return True if linear or acceptable non-linear"""
        if not self.linear:
            if len(self.unitList) > 1 or self.unitList[0].exp != 1:
                return False
        return True

    def compatStr(self):
        """Return string with reduced unit or linear compatability problem"""
        if self.checkLinear():
            return self.unitString(self.reducedList)
        return 'Cannot combine non-linear units'

    def convert(self, num, toGroup):
        """Return num of this group converted to toGroup"""
        if self.linear:
            num *= self.factor
        else:
            num = self.nonLinearCalc(num, 1) * self.factor
        if toGroup.linear:
            return num / toGroup.factor
        return toGroup.nonLinearCalc(num / toGroup.factor, 0)

    def nonLinearCalc(self, num, isFrom):
        """Return result of non-linear calculation"""
        x = num
        try:
            if self.unitList[0].toEqn:      # regular equations
                if isFrom:
                    return float(eval(self.unitList[0].fromEqn))
                return float(eval(self.unitList[0].toEqn))
            data = list(eval(self.unitList[0].fromEqn))  # extrapolation list
            if isFrom:
                data = [(float(group[0]), float(group[1])) for group in data]
            else:
                data = [(float(group[1]), float(group[0])) for group in data]
            data.sort()
            pos = len(data) - 1
            for i in range(len(data)):
                if num <= data[i][0]:
                    pos = i
                    break
            if pos == 0:
                pos = 1
            return (num-data[pos-1][0]) / float(data[pos][0]-data[pos-1][0]) \
                   * (data[pos][1]-data[pos-1][1]) + data[pos-1][1]
        except OverflowError:
            return 1e9999
        except:
            raise UnitDataError, 'Bad equation for %s' % self.unitList[0].name

    def convertStr(self, num, toGroup):
        """Return formatted string of converted number"""
        return self.formatNumStr(self.convert(num, toGroup))

    def formatNumStr(self, num):
        """Return num string formatted per options"""
        decPlcs = self.option.intData('DecimalPlaces', 0, UnitGroup.maxDecPlcs)
        if self.option.boolData('SciNotation'):
            return ('%%0.%dE' % decPlcs) % num
        if self.option.boolData('FixedDecimals'):
            return ('%%0.%df' % decPlcs) % num
        return ('%%0.%dG' % decPlcs) % num







import keyring
from getpass import getpass

def set_password(service, user, pw=None):
    print 'Set password:'
    if pw is None:
        pw = getpass()
        print 'Type password again (for verification):'
        if pw != getpass():
            print 'Error: passwords did not match. Try again.'
            set_password(service, user)
    keyring.set_password(service, user, pw)
    return pw


def password(service, user):
    pw = keyring.get_password(service, user)
    if pw is None:
        pw = set_password(service, user)
    return pw



#!/usr/bin/env python
"""
latin1_to_ascii -- The UNICODE Hammer -- AKA "The Stupid American"

This takes a UNICODE string and replaces Latin-1 characters with
something equivalent in 7-bit ASCII. This returns a plain ASCII string.
This function makes a best effort to convert Latin-1 characters into
ASCII equivalents. It does not just strip out the Latin1 characters.
All characters in the standard 7-bit ASCII range are preserved.
In the 8th bit range all the Latin-1 accented letters are converted to
unaccented equivalents. Most symbol characters are converted to
something meaningful. Anything not converted is deleted.

Background:

One of my clients gets address data from Europe, but most of their systems
cannot handle Latin-1 characters. With all due respect to the umlaut,
scharfes s, cedilla, and all the other fine accented characters of Europe,
all I needed to do was to prepare addresses for a shipping system.
After getting headaches trying to deal with this problem using Python's
built-in UNICODE support I gave up and decided to use some brute force.
This function converts all accented letters to their unaccented equivalents.
I realize this is dirty, but for my purposes the mail gets delivered.
"""

def latin1_to_ascii (unicrap):
    """This takes a UNICODE string and replaces Latin-1 characters with
        something equivalent in 7-bit ASCII. It returns a plain ASCII string.
        This function makes a best effort to convert Latin-1 characters into
        ASCII equivalents. It does not just strip out the Latin-1 characters.
        All characters in the standard 7-bit ASCII range are preserved.
        In the 8th bit range all the Latin-1 accented letters are converted
        to unaccented equivalents. Most symbol characters are converted to
        something meaningful. Anything not converted is deleted.
    """
    xlate={0xc0:'A', 0xc1:'A', 0xc2:'A', 0xc3:'A', 0xc4:'A', 0xc5:'A',
        0xc6:'Ae', 0xc7:'C',
        0xc8:'E', 0xc9:'E', 0xca:'E', 0xcb:'E',
        0xcc:'I', 0xcd:'I', 0xce:'I', 0xcf:'I',
        0xd0:'Th', 0xd1:'N',
        0xd2:'O', 0xd3:'O', 0xd4:'O', 0xd5:'O', 0xd6:'O', 0xd8:'O',
        0xd9:'U', 0xda:'U', 0xdb:'U', 0xdc:'U',
        0xdd:'Y', 0xde:'th', 0xdf:'ss',
        0xe0:'a', 0xe1:'a', 0xe2:'a', 0xe3:'a', 0xe4:'a', 0xe5:'a',
        0xe6:'ae', 0xe7:'c',
        0xe8:'e', 0xe9:'e', 0xea:'e', 0xeb:'e',
        0xec:'i', 0xed:'i', 0xee:'i', 0xef:'i',
        0xf0:'th', 0xf1:'n',
        0xf2:'o', 0xf3:'o', 0xf4:'o', 0xf5:'o', 0xf6:'o', 0xf8:'o',
        0xf9:'u', 0xfa:'u', 0xfb:'u', 0xfc:'u',
        0xfd:'y', 0xfe:'th', 0xff:'y',
        0xa1:'!', 0xa2:'{cent}', 0xa3:'{pound}', 0xa4:'{currency}',
        0xa5:'{yen}', 0xa6:'|', 0xa7:'{section}', 0xa8:'{umlaut}',
        0xa9:'{C}', 0xaa:'{^a}', 0xab:'<<', 0xac:'{not}',
        0xad:'-', 0xae:'{R}', 0xaf:'_', 0xb0:'{degrees}',
        0xb1:'{+/-}', 0xb2:'{^2}', 0xb3:'{^3}', 0xb4:"'",
        0xb5:'{micro}', 0xb6:'{paragraph}', 0xb7:'*', 0xb8:'{cedilla}',
        0xb9:'{^1}', 0xba:'{^o}', 0xbb:'>>',
        0xbc:'{1/4}', 0xbd:'{1/2}', 0xbe:'{3/4}', 0xbf:'?',
        0xd7:'*', 0xf7:'/'
        }

    r = ''
    for i in unicrap:
        if xlate.has_key(ord(i)):
            r += xlate[ord(i)]
        elif ord(i) >= 0x80:
            pass
        else:
            r += str(i)
    return r

if __name__ == '__main__':
    from sys import stdin, stdout
    for line in stdin:
        stdout.write(latin1_to_ascii(line))



# stdlib
import re, os, sys, traceback, warnings, webbrowser
import subprocess, tempfile, BaseHTTPServer
from functools import wraps
from StringIO import StringIO
from contextlib import contextmanager
from threading import Thread

try:
    from arsenal.passwords import *
except ImportError:
    pass

from arsenal.terminal import colors


class ddict(dict):
    """
    Variation on collections.defaultdict which allows default value callback to
    inspect missing key.
    """
    def __init__(self, f):
        self.f = f

    def __missing__(self, key):
        self[key] = c = self.f(key)
        return c


@contextmanager
def ignore_error(color='red'):
    try:
        yield
    except:
        etype, evalue, tb = sys.exc_info()
        tb = '\n'.join(traceback.format_exception(etype, evalue, tb))

        if color is not None:
            color = getattr(colors, color)
        else:
            color = '%s'

        print color % '*'*80
        print color % tb
        print color % '*'*80


class logger(object):

    def __init__(self, filename, clean=True, quiet=False):
        if clean:
            os.system('rm -f ' + filename)
        self.f = file(filename, 'wb')
        self.quiet = quiet

    def write(self, x):
        self.f.write(x)
        if not self.quiet:
            sys.__stdout__.write(x)

    @staticmethod
    def start(*args, **kw):
        """ redirect stdout and stderr to logger """
        log = logger(*args, **kw)
        sys.stderr = sys.stdout = log
        return log

def force(g):
    """ force evaluation of generator `g`. """
    @wraps(g)
    def wrap(*args, **kw):
        return list(g(*args, **kw))
    return wrap



'''
timv: put this utility somewhere else. I have several variants on it. Should
consolidate modifications...

def parse_sexpr(e):
    """
    Parse a string representing an s-expressions into lists-of-lists.

    based on implementation by George Sakkis
    http://mail.python.org/pipermail/python-list/2005-March/312004.html
    """
    e = re.compile('^\s*;.*?\n', re.M).sub('', e)  # remove comments

    es, stack = [], []
    for token in re.split(r'([()])|\s+', e):
        if token == '(':
            new = []
            if stack:
                stack[-1].append(new)
            else:
                es.append(new)
            stack.append(new)
        elif token == ')':
            try:
                stack.pop()
            except IndexError:
                raise ValueError("Unbalanced right parenthesis: %s" % e)
        elif token:
            try:
                stack[-1].append(token)
            except IndexError:
                raise ValueError("Unenclosed subexpression (near %s)" % token)
    return es
'''

'''
# by George Sakkis (gsakkis at rutgers.edu)
# http://mail.python.org/pipermail/python-list/2005-March/312004.html
def parse_sexpr(expression, multiple=False):
    "If multiple s-expressions expected as output, set multiple to True."
    subexpressions,stack = [],[]
    for token in re.split(r'([()])|\s+', expression):
        if token == '(':
            new = []
            if stack:
                stack[-1].append(new)
            else:
                subexpressions.append(new)
            stack.append(new)
        elif token == ')':
            try:
                stack.pop()
            except IndexError:
                raise ValueError("Unbalanced right parenthesis: %s" % expression)
        elif token:
            try:
                stack[-1].append(token)
            except IndexError:
                raise ValueError("Unenclosed subexpression (near %s)" % token)
    if stack:
        raise ValueError("Unbalanced left parenthesis: %s" % expression)
    if len(subexpressions) > 1 and not multiple:
        raise ValueError("Single s-expression expected (%d given)" % len(subexpressions))
    return subexpressions[0]
'''

def attn(msg):
    """ Display a dialog which is sure to get my attention. """
    import gtk
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window.connect("delete_event", lambda *x: False)
    window.connect("destroy", lambda *x: gtk.main_quit())
    window.set_border_width(10)
    button = gtk.Button(msg)
    button.connect_object("clicked", gtk.Widget.destroy, window)
    window.add(button)
    button.show()
    window.show()
    window.fullscreen()
    gtk.main()


## # borrowed from whoosh
## def find_object(name, blacklist=None, whitelist=None):
##     """Imports and returns an object given a fully qualified name.
##
##     >>> find_object("whoosh.analysis.StopFilter")
##     <class 'whoosh.analysis.StopFilter'>
##     """
##
##     if blacklist:
##         for pre in blacklist:
##             if name.startswith(pre):
##                 raise TypeError("%r: can't instantiate names starting with %r" % (name, pre))
##     if whitelist:
##         passes = False
##         for pre in whitelist:
##             if name.startswith(pre):
##                 passes = True
##                 break
##         if not passes:
##             raise TypeError("Can't instantiate %r" % name)
##
##     lastdot = name.rfind(".")
##
##     assert lastdot > -1, "Name %r must be fully qualified" % name
##     modname = name[:lastdot]
##     clsname = name[lastdot + 1:]
##
##     mod = __import__(modname, fromlist=[clsname])
##     cls = getattr(mod, clsname)
##     return cls


#def trace(f):
#    def wrap(*args, **kw):
#        print 'calling function: {f} with args: {args} kwargs: {kw}'.format(f=f, args=args, kw=kw)
#        y = f(*args, **kw)
#        print ' ->', y
#        return y
#    return wrap

## I'm not sure about this function yet..
## def deep_import(name):
##     """
##     A version of __import__ that works as expected when using the form
##     'module.name' (returns the object corresponding to 'name', instead of
##     'module').
##     """
##     mod = __import__(name)
##     components = name.split('.')
##     for comp in components[1:]:
##         mod = getattr(mod, comp)
##     return mod

'''
import difflib
def showdiff(old, new):
    d = difflib.Differ()
    lines = d.compare(old.lines(),new.lines())
    realdiff = False
    for l in lines:
        print l,
        if not realdiff and not l[0].isspace():
            realdiff = True
    return realdiff
'''


def piped():
    """ Returns piped input via stdin, else None. """
    return sys.stdin if not sys.stdin.isatty() else None


def highlighter(p, flags=0):
    pattern = re.compile('%s' % p, flags)
    return lambda x: pattern.sub(colors.bold % colors.yellow % colors.bg_red % r'\1', x)


def deprecated(use_instead=None):
    """
    This is a decorator which can be used to mark functions as deprecated.
    It will result in a warning being emitted when the function is used.
    """
    def wrapped(func):
        @wraps(func)
        def new_func(*args, **kwargs):
            message = "Call to deprecated function %s." % func.__name__
            if use_instead:
                message += " Use %s instead." % use_instead
            warnings.warn(message, category=DeprecationWarning, stacklevel=2)
            return func(*args, **kwargs)
        return new_func

    return wrapped


def browser(html):
    """
    Display html in the default web browser without creating a temp file.

    Instantiates a trivial http server and calls webbrowser.open with a URL
    to retrieve html from that server.
    """

    class RequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
        def do_GET(self):
            bufferSize = 1024*1024
            for i in xrange(0, len(html), bufferSize):
                self.wfile.write(html[i:i+bufferSize])

    server = BaseHTTPServer.HTTPServer(('127.0.0.1', 0), RequestHandler)
    webbrowser.open('http://127.0.0.1:%s' % server.server_port)
    server.handle_request()


def pager(s, pager='less'):
    """Use the pager passed in and send string s through it."""
    subprocess.Popen(pager, stdin=subprocess.PIPE).communicate(s)


def edit_with_editor(s=None):
    """
    Open os.environ['EDITOR'] and load in text s.

    Returns the text typed in the editor, after running strip().
    """
    # This is the first time I've used with!
    with tempfile.NamedTemporaryFile() as t:
        if s:
            t.write(str(s))
            t.seek(0)
        subprocess.call([os.environ.get('EDITOR', 'nano'), t.name])
        return t.read().strip()

#_________________________________________________________________________
#

# TODO: use a contextdecorator
@contextmanager
def ctx_redirect_io(f=None):
    r"""
    Usage example:
      >>> with ctx_redirect_io() as io_target:
      ...    print 'how is this for io?'
      >>> io_target.getvalue()
      'how is this for io?\n'
    """
    target = f or StringIO()

    # Redirect IO to target.
    original_stdout = sys.stdout
    try:
        sys.stdout = target
        # provide an entry point for the procedure we are wrapping as well
        # as a reference to target
        yield target
    finally:
        # Restore stdio
        sys.stdout = original_stdout


def redirect_io(f):
    """
    redirect all of the output to standard out to a StringIO instance,
    which can be accessed as an attribute of the function, f.io_target

    Usage Example:
        >>> @redirect_io
        ... def foo(x):
        ...    print x
        >>> foo('hello?')
        >>> print foo.io_target.getvalue()    # doctest:+NORMALIZE_WHITESPACE
        hello?
        >>>
    """
    @wraps(f)
    def wrap(*args, **kwargs):
        with ctx_redirect_io() as io_target:
            wrap.io_target = io_target
            return f(*args, **kwargs)
    return wrap

#______________________________________________________________________________
# Function decorators

def threaded(callback=lambda *args, **kwargs: None, daemonic=False):
    """Decorate  a function to run in its own thread and report the result
    by calling callback with it."""
    def innerDecorator(func):
        @wraps(func)
        def inner(*args, **kwargs):
            target = lambda: callback(func(*args, **kwargs))
            t = Thread(target=target)
            t.setDaemon(daemonic)
            t.start()
        return inner
    return innerDecorator

#_______________________________________________________________________________
#

if __name__ == '__main__':

    import doctest

    def run_tests():

        def test_redirect_io():
            @redirect_io
            def foo(x):
                print x
            msg = 'hello there?'
            foo(msg)
            assert str(foo.io_target.getvalue().strip()) == msg

        test_redirect_io()

    run_tests()
    print 'passed'

    doctest.testmod()

    attn('ATTENTION!')



import subprocess
import email.Utils

def safestr(obj, encoding='utf-8'):
    r"""
    Converts any given object to utf-8 encoded string. 
    
        >>> safestr('hello')
        'hello'
        >>> safestr(u'\u1234')
        '\xe1\x88\xb4'
        >>> safestr(2)
        '2'
    """
    if isinstance(obj, unicode):
        return obj.encode('utf-8')
    elif isinstance(obj, str):
        return obj
    elif hasattr(obj, 'next') and hasattr(obj, '__iter__'): # iterator
        return itertools.imap(safestr, obj)
    else:
        return str(obj)


def sendmail(from_address, to_address, subject, message, **kw):
    """
    Sends the email message `message` with mail and envelope headers for from 
    `from_address_` to `to_address` with `subject`. Additional email headers
    can be specified with the dictionary  `headers.

    Uses `/usr/sbin/sendmail`, (the typical location for the sendmail binary).
    """
        
    cc = kw.get('cc', [])
    bcc = kw.get('bcc', [])
    
    def listify(x):
        if not isinstance(x, list):
            return [safestr(x)]
        else:
            return [safestr(a) for a in x]

    from_address = safestr(from_address)

    to_address = listify(to_address)
    cc = listify(cc)
    bcc = listify(bcc)

    recipients = to_address + cc + bcc
    
    headers = {
      'MIME-Version': '1.0',
      'Content-Type': 'text/plain; charset=UTF-8',
      'Content-Disposition': 'inline',
      'From': from_address,
      'To': ", ".join(to_address),
      'Subject': subject
    }

    if cc:
        headers['Cc'] = ", ".join(cc)    

    from_address = email.Utils.parseaddr(from_address)[1]
    recipients = [email.Utils.parseaddr(r)[1] for r in recipients]
    message = ('\n'.join([safestr('%s: %s' % x) for x in headers.iteritems()])
               + "\n\n" +  safestr(message))

    sendmail = '/usr/sbin/sendmail'
    
    assert not from_address.startswith('-'), 'security'
    for r in recipients:
        assert not r.startswith('-'), 'security'

    p = subprocess.Popen(['/usr/sbin/sendmail', '-f', from_address] + recipients, stdin=subprocess.PIPE)
    p.stdin.write(message)
    p.stdin.close()
    p.wait()


if __name__ == '__main__':
    sendmail('test@herman.local', 'tim.f.vieira@gmail.com', 'this is a test', 'See subject.')



from heapq import heapify, heappush, heappop

class OrderedSet(object):
    """
    Set which remembers insertion ordering allowed iteration while changing size
    and determinism.
    """
    __slots__ = 'set', 'list'
    def __init__(self):
        self.set = set()
        self.list = []
    def __contains__(self, item):
        return item in self.set
    def __iter__(self):
        return iter(self.list)
    def add(self, item):
        if item not in self.set:
            self.set.add(item)
            self.list.append(item)


class prioritydict(dict):
    """Dictionary that can be used as a priority queue.

    Keys of the dictionary are items to be put into the queue, and values
    are their respective priorities. All dictionary methods work as expected.
    The advantage over a standard heapq-based priority queue is
    that priorities of items can be efficiently updated (amortized O(1))
    using code as 'thedict[item] = new_priority.'

    The 'smallest' method can be used to return the object with lowest
    priority, and 'pop_smallest' also removes it.

    The 'sorted_iter' method provides a destructive sorted iterator.

    This implemented is based on:

      Matteo Dell'Amico's implementation
      http://code.activestate.com/recipes/522995-priority-dict-a-priority-queue-with-updatable-prio/

         which is based on David Eppstein's implementation
         http://code.activestate.com/recipes/117228/

    """

    def __init__(self, *args, **kwargs):
        super(prioritydict, self).__init__(*args, **kwargs)
        self._heap = None
        self._rebuild_heap()

    def _rebuild_heap(self):
        self._heap = [(v, k) for k, v in self.iteritems()]
        heapify(self._heap)

    def pop_smallest(self):
        """
        Return the item with the lowest priority and remove it.

        Raises IndexError if the object is empty.
        """

        # Implementation note: Since we don't eagerly remove an element when
        # it's priority changes, we need to filter our pops to make sure the
        # priority isn't stale.

        heap = self._heap
        v, k = heappop(heap)
        while k not in self or self[k] != v:   # while `k` is stale
            v, k = heappop(heap)
        del self[k]
        return k

    def __setitem__(self, key, val):
        # We are not going to remove the previous value from the heap, since
        # this would have a cost O(n).
        super(prioritydict, self).__setitem__(key, val)

        if len(self._heap) < 2 * len(self):
            heappush(self._heap, (val, key))
        else:
            # When the heap grows larger than 2 * len(self), we rebuild it from
            # scratch to avoid wasting too much memory.
            self._rebuild_heap()

    setdefault = None
    update = None



from __future__ import division
import numpy as np
from numpy import array, exp, log, dot, abs as np_abs, multiply


def cumavg(x):
    """
    Cumulative average.

    >>> cumavg([1,2,3,4,5])
    array([ 1. ,  1.5,  2. ,  2.5,  3. ])

    """
    return np.cumsum(x) / np.arange(1.0, len(x)+1)


def normalize_zscore(data):
    """
    Shift and rescale data to be zero-mean and unit-variance along axis 0.
    """
    shift = data.mean(axis=0)
    rescale = data.std(axis=0)
    rescale[rescale == 0] = 1.0   # avoid divide by zero
    return (data - shift) / rescale


def normalize_interval(data):
    """
    Shift and rescale data so that it lies in the range [0,1].
    """
    shift = data.min(axis=0)
    rescale = data.max(axis=0) - shift
    rescale[rescale == 0] = 1.0  # avoid divide by zero
    return (data - shift) / rescale


def normalize(p):
    return p / p.sum()


def lidstone(p, delta):
    """
    Lidstone smoothing is a generalization of Laplace smoothing.
    """
    return normalize(p + delta)

'''
def logsumexp(a):
    """
    Compute the log of the sum of exponentials of an array ``a``, :math:`\log(\exp(a_0) + \exp(a_1) + ...)`
    """
    b = a.max()
    return b + log((exp(a-b)).sum())
'''

# based on implementation from scikits-learn
def logsumexp(arr, axis=0):
    """Computes the sum of arr assuming arr is in the log domain.

    Returns log(sum(exp(arr))) while minimizing the possibility of
    over/underflow.

    Examples
    --------

    >>> a = np.arange(10)
    >>> log(sum(exp(a)))
    9.4586297444267107
    >>> logsumexp(a)
    9.4586297444267107
    """
    arr = np.asarray(arr)
    arr = np.rollaxis(arr, axis)
    # Use the max to normalize, as with the log this is what accumulates the
    # less errors
    vmax = arr.max(axis=0)
    out = log(exp(arr - vmax).sum(axis=0))
    out += vmax
    return out


def exp_normalize(x, T=1.0):
    """
    >>> x = [1, -10, 100, .5]
    >>> exp_normalize(x)
    array([  1.01122149e-43,   1.68891188e-48,   1.00000000e+00,
             6.13336839e-44])
    >>> exp(x) / exp(x).sum()
    array([  1.01122149e-43,   1.68891188e-48,   1.00000000e+00,
             6.13336839e-44])
    """
    y = array(x)      # creates copy
    y /= T
    y -= y.max()
    y = exp(y)
    y /= y.sum()
    return y


log_of_2 = log(2)

def entropy(p):
    "Entropy of a discrete random variable with distribution `p`"
    assert len(p.shape) == 1
    p = p[p.nonzero()]
    return -dot(p, log(p)) / log_of_2


def kl_divergence(p, q):
    """ Compute KL divergence of two vectors, K(p || q).
    NOTE: If any value in q is 0.0 then the KL-divergence is infinite.
    """
    return dot(p, log(p / q)) / log_of_2


# KL(p||q) = sum_i p[i] log(p[i] / q[i])
#          = sum_i p[i] (log p[i] - log q[i])
#          = sum_i p[i] log p[i] - sum_i p[i] log(q[i])
#          = Entropy(p) + CrossEntropy(p,q)

# timv: do we need to specify all three things?
#
#  P(x) = sum_y P(x,y) by law of total probabilty
def mutual_information(joint):
    """
    Mutual Information

    MI(x,y) = KL( p(x,y) || p(x) p(y) )

    joint = p(x,y)
    p = p(x)
    q = q(y)

    relationships:
      MI(x,y) is the expected PMI(x,y) wrt p(x,y)
      MI(x,y) = KL( p(x,y) || p(x) p(y) )

    properties:
      MI(X,Y) = MI(Y,X) is symmetric
    """
    # we can compute px and py from joint by applying the law of total probability
    px = joint.sum(axis=1)
    py = joint.sum(axis=0)
    independent = multiply.outer(px, py)
    assert joint.shape == independent.shape
    return kl_divergence(array(joint.flat), array(independent.flat))


def cross_entropy(p, q):
    """ Cross Entropy of two vectors,

    CE(p,q) = - \sum_i p[i] log q[i]

    Relationship to KL-divergence:

      CE(p,q) = entropy(p) + KL(p||q)
    """
    assert len(p) == len(q)
    p = p[p > 0]
    return -dot(p, log(q)) / log_of_2


def assert_isdistr(p):
    assert (p >= 0).all()
    assert (p <= 1).all()
    assert abs(p.sum() - 1.0) < 0.000001


def equal(a, b, tol=1e-10):
    "L_{\inf}(a - b) < tol"
    return inf_norm(a,b) < tol


def inf_norm(a, b):
    return np_abs(a - b).max()


def assert_equal(a, b, tol=1e-10):
    err = inf_norm(a,b)
    assert err < tol, 'error = %s >= tolerance (%s)' % (err, tol)


if __name__ == '__main__':
    #import doctest; doctest.testmod()

    def tests():

        # Entropy tests
        assert entropy(array((0.5, 0.5))) == 1.0
        assert abs(entropy(array((0.75, 0.25))) - 0.8112781244) < 1e-10
        assert abs(entropy(array((0.1, 0.1, 0.8))) - 0.9219280948) < 1e-10

        # KL-divergence tests
        assert kl_divergence(array((0.5, 0.5)), array((0.5, 0.5))) == 0.0

        # KL, Entropy, and Cross Entropy relationship
        p = array([0.5, 0.5])
        q = array([0.4, 0.6])
        assert_equal(cross_entropy(p, q), (entropy(p) + kl_divergence(p, q)))

        # Normalize tests
        assert_equal(array([0.5, 0.5]), normalize(array([2, 2])))

        # Mutual Information tests
        #Pjoint = array([[0.25, 0.25], [0.25, 0.25]])

        Pjoint = normalize(np.random.rand(4,10))
        assert_equal(mutual_information(Pjoint),
                     mutual_information(Pjoint.T))

        def mi_independent_is_zero(px, py):
            assert_equal(mutual_information(multiply.outer(px, py)), 0.0)

        mi_independent_is_zero(p, q)
        mi_independent_is_zero(array([0.1, 0.1, 0.2, 0.6]),   # non-square joint
                               array([0.9, 0.1]))


        print 'passed tests.'

    tests()



def k_subsets_i(n, k):
    """ Yield each subset of size k from the set of integers 0 .. n - 1 """
    # check base cases
    if k == 0 or n < k:
        yield []
    elif n == k:
        yield range(n) 
    else:
        # Use recursive formula based on binomial coeffecients:
        # choose(n, k) = choose(n - 1, k - 1) + choose(n - 1, k)
        for s in k_subsets_i(n - 1, k - 1):
            s.append(n - 1)
            yield s
        for s in k_subsets_i(n - 1, k):
            yield s

def k_subsets(s, k):
    """ Yield all subsets of size k from s """
    s = list(s)
    n = len(s)
    for k_set in k_subsets_i(n, k):
        yield [s[i] for i in k_set]

def powerset(S):
    # number of variables
    n = len(S)
    # choose all subsets of size k from variables
    for k in range(0,n+1):
        for s in k_subsets(S, k):
            yield list(s)

# __________________________________________________________________________
"""
xpermutations.py
Generators for calculating
a) the permutations of a sequence and
b) the combinations and selections of a number of elements from a sequence.
"""

def xcombinations(items, n):
    if n==0: yield []
    else:
        for i in xrange(len(items)):
            for cc in xcombinations(items[:i]+items[i+1:],n-1):
                yield [items[i]]+cc

def xuniqueCombinations(items, n):
    if n==0: yield []
    else:
        for i in xrange(len(items)):
            for cc in xuniqueCombinations(items[i+1:],n-1):
                yield [items[i]]+cc
            
def xselections(items, n):
    if n==0: yield []
    else:
        for i in xrange(len(items)):
            for ss in xselections(items, n-1):
                yield [items[i]]+ss

def xpermutations(items):
    return xcombinations(items, len(items))

if __name__=="__main__":
    print "Permutations of 'love'"
    for p in xpermutations(['l','o','v','e']): print ''.join(p)

    print
    print "Combinations of 2 letters from 'love'"
    for c in xcombinations(['l','o','v','e'],2): print ''.join(c)

    print
    print "Unique Combinations of 2 letters from 'love'"
    for uc in xuniqueCombinations(['l','o','v','e'],2): print ''.join(uc)

    print
    print "Selections of 2 letters from 'love'"
    for s in xselections(['l','o','v','e'],2): print ''.join(s)

    print
    print map(''.join, list(xpermutations('done')))



#!/usr/bin/env python

import sys, os.path
try:
    from __main__ import dataFilePath
except ImportError:
    dataFilePath = None

import unitatom

class UnitDataError(Exception):
    pass

class UnitData(dict):
    def __init__(self):
        dict.__init__(self)
        self.sortedKeys = []
        self.typeList = []

    def findDataFile(self, pathList):
        """Search paths for file, return line list or None"""
        import data
        return data.data.split('\n')

    def readData(self):
        """Read all unit data from file"""
        modPath = os.path.abspath(sys.path[0])
        pathList = [dataFilePath, os.path.join(modPath, '../data/'), modPath]
        lines = self.findDataFile(filter(None, pathList))
        for i in range(len(lines)):     # join continuation lines
            delta = 1
            while lines[i].rstrip().endswith('\\'):
                lines[i] = ''.join([lines[i].rstrip()[:-1], lines[i+delta]])
                lines[i+delta] = ''
                delta += 1
        units = [unitatom.UnitAtom(line) for line in lines if line.split('#', 1)[0].strip()]   # remove comment lines
        typeText = ''
        for unit in units:               # find & set headings
            if unit.name.startswith('['):
                typeText = unit.name[1:-1].strip()
                self.typeList.append(typeText)
            unit.typeName = typeText
        units = [unit for unit in units if unit.equiv]  # keep valid units
        for unit in units:
            self[unit.name.lower().replace(' ', '')] = unit
        self.sortedKeys = self.keys()
        self.sortedKeys.sort()
        if len(self.sortedKeys) < len(units):
            raise UnitDataError, 'Duplicate unit names found'

    def findPartialMatch(self, text):
        """Return first partially matching unit or None"""
        text = text.lower().replace(' ', '')
        if not text:
            return None
        for name in self.sortedKeys:
            if name.startswith(text):
                return self[name]
        return None

    def findSortPos(self, text):
        """Return unit whose abbrev comes immediately after text"""
        text = text.lower().replace(' ', '')
        for name in self.sortedKeys:
            if text <= name:
                return self[name]
        return self[self.sortedKeys[-1]]

    def filteredList(self, type='', srchStr=''):
        """Return list of units matching type and search string,
           if given"""
        units = self.values()
        if type:
            units = [unit for unit in units if unit.typeName == type]
        if srchStr.strip():
            srchWords = [word.lower() for word in srchStr.split()]
            srchUnits = []
            for unit in units:
                if unit.matchWords(srchWords):
                    srchUnits.append(unit)
            units = srchUnits
        return units




requires_number_arg = [
    ('APARTMENT', 'APT'),
    ('BUILDING', 'BLDG'),
    ('DEPARTMENT', 'DEPT'),
    ('FLOOR', 'FL'),
    ('HANGAR', 'HNGR'),
    ('LOT', 'LOT'),
    ('PIER', 'PIER'),
    ('ROOM', 'RM'),
    ('SLIP', 'SLIP'),
    ('SPACE', 'SPC'),
    ('STOP', 'STOP'),
    ('SUITE', 'STE'),
    ('TRAILER', 'TRLR'),
    ('UNIT', 'UNIT'),
]

# Does not require secondary range number to follow
do_not_require_number_arg = [
    ('BASEMENT', 'BSMT'),
    ('FRONT', 'FRNT'),
    ('LOBBY', 'LBBY'),
    ('LOWER', 'LOWR'),
    ('OFFICE', 'OFC'),
    ('PENTHOUSE', 'PH'),
    ('REAR', 'REAR'),
    ('SIDE', 'SIDE'),
    ('UPPER', 'UPPR'),
]




import os
import sys

from time import sleep
from threading import Thread

_win = sys.platform == "win32"

# TODO: make this a more general purpose utility
class watch(Thread):
    """
    Daemon thread with watches a file for changes
    """
    def __init__(self, mod, name=None, pause=1, verbose=True):
        Thread.__init__(self)
        self.filename = mod.__file__   # TODO: consider using debug.edit get_filename
        self.name = name or mod.__name__
        self.mod = mod
        self.pause = pause
        self.verbose = verbose
        self.setDaemon(True)
        self.start()
        self.mtimes = {}

    def run(self):
        if self.filename.endswith('.pyc') or self.filename.endswith('.pyo'):
            self.filename = self.filename[0:-1]
        if self.verbose:
            print 'autoreload module:', self.name
            print 'watching file:', self.filename
        while True:
            if self.file_changed(self.filename):
                try:
                    if self.verbose:
                        print 'reloading', self.name
                    globals()[self.name] = reload(self.mod)
                except Exception as e:
                    print 'error in:', self.filename, '... ignoring reload.'
                    print e
            sleep(self.pause)
        if self.verbose:
            print 'thread done...'

    def file_changed(self, filename):
        if not os.path.exists(filename):
            sys.stderr.write("file '%s' doens't exist..." % filename)
            return False
        stat = os.stat(filename)
        mtime = stat.st_mtime
        if _win:
            mtime -= stat.st_ctime
        if filename not in self.mtimes:
            self.mtimes[filename] = mtime
            return False
        if mtime != self.mtimes[filename]:
            self.mtimes.clear()
            return True
        return False


if __name__ == '__main__':

    def test():
        old = "old message"
        new = "new message :-)"
        function = 'f = lambda : %r'
        filename = 'watch_me.py'

        try:
            with file(filename, 'wb') as f:
                f.write(function % old)

            import watch_me
            watch(watch_me, pause=0.1)

            assert watch_me.f() == old

            # wait a few seconds to make a change
            sleep(2)

            # make a change
            with file(filename, 'wb') as f:
                f.write(function % new)

            # wait a few seconds
            sleep(4)

            assert watch_me.f() == new

        finally:

            print 'removing temporary files...'
            try:
                os.remove('watch_me.py')
                os.remove('watch_me.pyc')
            except OSError:
                pass

        print 'done.'
    test()
